"""Test check for classes extending an Enum class."""
from enum import IntFlag, auto

class DisjointFlags(IntFlag):
    """Class with flags that do not overlap"""
    X = 1
    W = 2
    R = 4
    S = auto()

class ExplicitUnionFlags(IntFlag):
    """Class with flags that overlap using explicit union syntax"""
    X = 1
    W = 2
    R = 4
    RO = 4
    RW = R | W

class SubclassUnionFlags(ExplicitUnionFlags):  # [invalid-enum-extension]
    """Class with flags that overlap a superclass"""
    RWX = 7

class ImplicitUnionFlags(IntFlag):
    """Class with flags that overlap implicitly"""
    A = 1
    DUPLICATE = 1
    B = 2
    C = 3  # [implicit-flag-alias]
    D = 5  # [implicit-flag-alias]
    E = 8
    F = 17  # [implicit-flag-alias]
    G = 17  # [implicit-flag-alias]
    TRIPLICATE = 1

    def irrelevant(self):
        """Irrelevant method definition"""
        return

class ReverseOrderFlags(IntFlag):
    """Class with flags that overlap and are declared in descending order"""
    A = 5  # [implicit-flag-alias]
    B = 1

class SharedBitsWithoutDeclaration(IntFlag):
    """Class with flags that share a bit, but no value is defined using that bit"""
    A = 3
    B = 5  # [implicit-flag-alias]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Test for the 'pylint-config' utils."""


import pytest
from pytest import CaptureFixture, MonkeyPatch

from pylint.config._pylint_config.utils import get_and_validate_format


def test_retrying_user_input_validation(
    monkeypatch: MonkeyPatch, capsys: CaptureFixture[str]
) -> None:
    """Check that we retry after a wrong answer."""
    # Set the answers needed for the input() calls
    answers = iter(["A", "B", "EXIT", "EXIT()"])
    monkeypatch.setattr("builtins.input", lambda x: next(answers))

    with pytest.raises(SystemExit):
        get_and_validate_format()
    captured = capsys.readouterr()
    assert (
        captured.out
        == """Answer should be one of i, ini, t, toml.
Type 'exit()' if you want to exit the program.
Answer should be one of i, ini, t, toml.
Type 'exit()' if you want to exit the program.
Answer should be one of i, ini, t, toml.
Type 'exit()' if you want to exit the program.
Stopping 'pylint-config'.
"""
    )



# pylint: disable=missing-docstring, import-error, yield-outside-function
import factory
from magic import shazam, turbogen

yield 1

def bad(generator):
    for item in generator:  # [use-yield-from]
        yield item


def out_of_names():
    for item in turbogen():  # [use-yield-from]
        yield item


def good(generator):
    for item in generator:
        shazam()
        yield item


def yield_something():
    yield 5


def yield_attr():
    for item in factory.gen():  # [use-yield-from]
        yield item


def yield_attr_nested():
    for item in factory.kiwi.gen():  # [use-yield-from]
        yield item


def yield_expr():
    for item in [1, 2, 3]:  # [use-yield-from]
        yield item


def for_else_yield(gen, something):
    for item in gen():
        if shazam(item):
            break
    else:
        yield something


# yield from is not supported in async functions, so the following are fine

async def async_for_yield(agen):
    async for item in agen:
        yield item


async def async_yield(agen):
    for item in agen:
        yield item



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import sys
from unittest import mock

import pytest

from pylint.checkers import BaseChecker
from pylint.constants import WarningScope
from pylint.exceptions import InvalidMessageError
from pylint.lint.pylinter import PyLinter
from pylint.message import MessageDefinition


@pytest.mark.parametrize(
    "msgid,expected",
    [
        ("Q1234", "Bad message type Q in 'Q1234'"),
        ("W12345", "Invalid message id 'W12345'"),
    ],
)
def test_create_invalid_message_type(msgid: str, expected: str) -> None:
    checker_mock = mock.Mock(name="Checker")
    checker_mock.name = "checker"

    with pytest.raises(InvalidMessageError) as invalid_message_error:
        MessageDefinition.check_msgid(msgid)
    with pytest.raises(InvalidMessageError) as other_invalid_message_error:
        MessageDefinition(checker_mock, msgid, "msg", "descr", "symbol", "scope")
    assert str(invalid_message_error.value) == expected
    assert str(other_invalid_message_error.value) == expected


class FalseChecker(BaseChecker):
    def __init__(self) -> None:
        super().__init__(PyLinter())

    name = "FalseChecker"
    msgs = {
        "W1234": ("message one", "msg-symbol-one", "msg description"),
        "W1235": (
            "message two",
            "msg-symbol-two",
            "msg description",
            {"old_names": [("W1230", "msg-symbol-one")]},
        ),
    }


class TestMessagesDefinition:
    @staticmethod
    def assert_with_fail_msg(
        msg: MessageDefinition,
        expected: bool = True,
        py_version: tuple[int, ...] | sys._version_info = sys.version_info,
    ) -> None:
        fail_msg = (
            f"With minversion='{msg.minversion}' and maxversion='{msg.maxversion}',"
            f" and the py-version option being {py_version} "
            "the message should{}be emitable"
        )
        if expected:
            assert msg.may_be_emitted(py_version), fail_msg.format(" ")
        else:
            assert not msg.may_be_emitted(py_version), fail_msg.format(" not ")

    @staticmethod
    def get_message_definition() -> MessageDefinition:
        return MessageDefinition(
            FalseChecker(),
            "W1234",
            "message",
            "description",
            "msg-symbol",
            WarningScope.NODE,
        )

    def test_may_be_emitted_default(self) -> None:
        major = sys.version_info.major
        minor = sys.version_info.minor
        msg = self.get_message_definition()
        self.assert_with_fail_msg(msg, expected=True)
        msg.minversion = (major, minor - 1)
        msg.maxversion = (major, minor + 1)
        self.assert_with_fail_msg(msg, expected=True)
        msg.minversion = (major, minor + 1)
        self.assert_with_fail_msg(msg, expected=False)
        msg.minversion = (major, minor - 1)
        self.assert_with_fail_msg(msg, expected=True)
        msg.maxversion = (major, minor - 1)
        self.assert_with_fail_msg(msg, expected=False)

    def test_may_be_emitted_py_version(self) -> None:
        msg = self.get_message_definition()
        self.assert_with_fail_msg(msg, expected=True, py_version=(3, 2))

        msg.maxversion = (3, 5)
        self.assert_with_fail_msg(msg, expected=True, py_version=(3, 2))
        self.assert_with_fail_msg(msg, expected=False, py_version=(3, 5))
        self.assert_with_fail_msg(msg, expected=False, py_version=(3, 6))

        msg.maxversion = None
        msg.minversion = (3, 9)
        self.assert_with_fail_msg(msg, expected=True, py_version=(3, 9))
        self.assert_with_fail_msg(msg, expected=True, py_version=(3, 10))
        self.assert_with_fail_msg(msg, expected=False, py_version=(3, 8))

    def test_repr(self) -> None:
        msg = self.get_message_definition()
        repr_str = str([msg, msg])
        assert "W1234" in repr_str
        assert "msg-symbol" in repr_str
        expected = "[MessageDefinition:msg-symbol-one (W1234), MessageDefinition:msg-symbol-two (W1235)]"
        assert str(FalseChecker().messages) == expected

    def test_str(self) -> None:
        msg = self.get_message_definition()
        str_msg = str(msg)
        assert "W1234" in str_msg
        assert "msg-symbol" in str_msg
        expected = """MessageDefinition:msg-symbol-one (W1234):
message one msg description"""
        assert str(FalseChecker().messages[0]) == expected

    def test_format_help(self) -> None:
        msg = self.get_message_definition()
        major = sys.version_info.major
        minor = sys.version_info.minor
        msg.minversion = (major, minor - 1)
        msg.maxversion = (major, minor + 1)
        format_str_checker_ref = msg.format_help(checkerref=False)
        format_str = msg.format_help(checkerref=True)
        assert str(minor - 1) in format_str
        assert str(major + 1) in format_str_checker_ref
        expected_format_help = """:msg-symbol-one (W1234): *message one*
  msg description"""
        assert FalseChecker().messages[0].format_help() == expected_format_help



"""Test generic alias support for stdlib types (added in PY39).

In type annotation context, they can be used with postponed evaluation enabled,
starting with PY37.
"""
# flake8: noqa
# pylint: disable=missing-docstring,pointless-statement,invalid-name
# pylint: disable=too-few-public-methods,multiple-statements,line-too-long
from __future__ import annotations

import abc
import collections
import collections.abc
import contextlib
import re


# ----- unsubscriptable (even with postponed evaluation) -----
# special
tuple[int, int]  # [unsubscriptable-object]
type[int]  # [unsubscriptable-object]
collections.abc.Callable[[int], str]  # [unsubscriptable-object]

# builtins
dict[int, str]  # [unsubscriptable-object]
list[int]  # [unsubscriptable-object]
set[int]  # [unsubscriptable-object]
frozenset[int]  # [unsubscriptable-object]

# collections
collections.defaultdict[int, str]  # [unsubscriptable-object]
collections.OrderedDict[int, str]  # [unsubscriptable-object]
collections.ChainMap[int, str]  # [unsubscriptable-object]
collections.Counter[int]  # [unsubscriptable-object]
collections.deque[int]  # [unsubscriptable-object]

# collections.abc
collections.abc.Set[int]  # [unsubscriptable-object]
collections.abc.Collection[int]  # [unsubscriptable-object]
collections.abc.Container[int]  # [unsubscriptable-object]
collections.abc.ItemsView[int, str]  # [unsubscriptable-object]
collections.abc.KeysView[int]  # [unsubscriptable-object]
collections.abc.Mapping[int, str]  # [unsubscriptable-object]
collections.abc.MappingView[int]  # [unsubscriptable-object]
collections.abc.MutableMapping[int, str]  # [unsubscriptable-object]
collections.abc.MutableSequence[int]  # [unsubscriptable-object]
collections.abc.MutableSet[int]  # [unsubscriptable-object]
collections.abc.Sequence[int]  # [unsubscriptable-object]
collections.abc.ValuesView[int]  # [unsubscriptable-object]

collections.abc.Iterable[int]  # [unsubscriptable-object]
collections.abc.Iterator[int]  # [unsubscriptable-object]
collections.abc.Generator[int, None, None]  # [unsubscriptable-object]
collections.abc.Reversible[int]  # [unsubscriptable-object]

collections.abc.Coroutine[list[str], str, int]  # [unsubscriptable-object,unsubscriptable-object]
collections.abc.AsyncGenerator[int, None]  # [unsubscriptable-object]
collections.abc.AsyncIterable[int]  # [unsubscriptable-object]
collections.abc.AsyncIterator[int]  # [unsubscriptable-object]
collections.abc.Awaitable[int]  # [unsubscriptable-object]

# contextlib
contextlib.AbstractContextManager[int]  # [unsubscriptable-object]
contextlib.AbstractAsyncContextManager[int]  # [unsubscriptable-object]

# re
re.Pattern[str]  # [unsubscriptable-object]
re.Match[str]  # [unsubscriptable-object]


# unsubscriptable types
collections.abc.Hashable
collections.abc.Sized
collections.abc.Hashable[int]  # [unsubscriptable-object]
collections.abc.Sized[int]  # [unsubscriptable-object]

# subscriptable with Python 3.9
collections.abc.ByteString[int]  # [unsubscriptable-object]


# Missing implementation for 'collections.abc' derived classes
class DerivedHashable(collections.abc.Hashable):  # [abstract-method]  # __hash__
    pass

class DerivedIterable(collections.abc.Iterable[int]):  # [unsubscriptable-object]
    pass

class DerivedCollection(collections.abc.Collection[int]):  # [unsubscriptable-object]
    pass


# No implementation required for 'builtins' and 'collections' types
class DerivedList(list[int]):  # [unsubscriptable-object]
    pass

class DerivedSet(set[int]):  # [unsubscriptable-object]
    pass

class DerivedOrderedDict(collections.OrderedDict[int, str]):  # [unsubscriptable-object]
    pass

class DerivedListIterable(list[collections.abc.Iterable[int]]):  # [unsubscriptable-object,unsubscriptable-object]
    pass


# Multiple generic base classes
class DerivedMultiple(collections.abc.Sized, collections.abc.Hashable):  # [abstract-method,abstract-method]
    pass

class CustomAbstractCls1(abc.ABC):
    pass
class CustomAbstractCls2(collections.abc.Sized, collections.abc.Iterable[CustomAbstractCls1]):  # [abstract-method,unsubscriptable-object]  # __len__
    pass
class CustomImplementation(CustomAbstractCls2):  # [abstract-method]  # __len__
    pass



# ----- subscriptable (with postponed evaluation) -----
# special
var_tuple: tuple[int, int]
var_type: type[int]
var_callable: collections.abc.Callable[[int], str]

# builtins
var_dict: dict[int, str]
var_list: list[int]
var_set: set[int]
var_frozenset: frozenset[int]

# collections
var_defaultdict: collections.defaultdict[int, str]
var_OrderedDict: collections.OrderedDict[int, str]
var_ChainMap: collections.ChainMap[int, str]
var_Counter: collections.Counter[int]
var_deque: collections.deque[int]

# collections.abc
var_abc_set: collections.abc.Set[int]
var_abc_collection: collections.abc.Collection[int]
var_abc_container: collections.abc.Container[int]
var_abc_ItemsView: collections.abc.ItemsView[int, str]
var_abc_KeysView: collections.abc.KeysView[int]
var_abc_Mapping: collections.abc.Mapping[int, str]
var_abc_MappingView: collections.abc.MappingView[int]
var_abc_MutableMapping: collections.abc.MutableMapping[int, str]
var_abc_MutableSequence: collections.abc.MutableSequence[int]
var_abc_MutableSet: collections.abc.MutableSet[int]
var_abc_Sequence: collections.abc.Sequence[int]
var_abc_ValuesView: collections.abc.ValuesView[int]

var_abc_Iterable: collections.abc.Iterable[int]
var_abc_Iterator: collections.abc.Iterator[int]
var_abc_Generator: collections.abc.Generator[int, None, None]
var_abc_Reversible: collections.abc.Reversible[int]

var_abc_Coroutine: collections.abc.Coroutine[list[str], str, int]
var_abc_AsyncGenerator: collections.abc.AsyncGenerator[int, None]
var_abc_AsyncIterable: collections.abc.AsyncIterable[int]
var_abc_AsyncIterator: collections.abc.AsyncIterator[int]
var_abc_Awaitable: collections.abc.Awaitable[int]

# contextlib
var_ContextManager: contextlib.AbstractContextManager[int]
var_AsyncContextManager: contextlib.AbstractAsyncContextManager[int]

# re
var_re_Pattern: re.Pattern[str]
var_re_Match: re.Match[str]


# unsubscriptable types
var_abc_Hashable: collections.abc.Hashable
var_abc_Sized: collections.abc.Sized
var_abc_Hashable2: collections.abc.Hashable[int]  # string annotations aren't checked
var_abc_Sized2: collections.abc.Sized[int]  # string annotations aren't checked

# subscriptable with Python 3.9
var_abc_ByteString: collections.abc.ByteString[int]


# Generic in type stubs only -> string annotations aren't checked
class A:
    ...

var_a1: A[str]  # string annotations aren't checked
var_a2: "A[str]"  # string annotations aren't checked
class B(A[str]):  # [unsubscriptable-object]
    ...



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

from collections.abc import ValuesView
from pathlib import Path

import pytest

from pylint.exceptions import InvalidMessageError, UnknownMessageError
from pylint.message.message_definition import MessageDefinition
from pylint.message.message_id_store import MessageIdStore
from pylint.testutils._run import _Run as Run

EMPTY_FILE = str(Path(__file__).parent.parent.resolve() / "regrtest_data" / "empty.py")


def test_len_str(msgid_store: MessageIdStore, msgids: dict[str, str]) -> None:
    assert len(msgid_store) == len(msgids)
    str_result = str(msgid_store)
    assert "MessageIdStore: [" in str_result
    assert "  - W1234 (warning-symbol)" in str_result
    assert "  - W1235 (warning-symbol-two)" in str_result
    assert "  - C1234 (convention-symbol)" in str_result
    assert "  - E1234 (error-symbol)" in str_result
    assert "]" in str_result


def test_get_message_ids(msgid_store: MessageIdStore, msgids: dict[str, str]) -> None:
    """We can get message id even with capitalization problem."""
    msgid = next(iter(msgids.keys()))
    msgids_result = msgid_store.get_active_msgids(msgid.lower())
    assert len(msgids_result) == 1
    assert msgid == msgids_result[0]


def test_get_message_ids_not_existing(empty_msgid_store: MessageIdStore) -> None:
    with pytest.raises(UnknownMessageError) as error:
        w9876 = "W9876"
        empty_msgid_store.get_active_msgids(w9876)
    assert w9876 in str(error.value)


def test_register_message_definitions(
    empty_msgid_store: MessageIdStore,
    message_definitions: ValuesView[MessageDefinition],
) -> None:
    number_of_msgid = len(message_definitions)
    for message_definition in message_definitions:
        empty_msgid_store.register_message_definition(
            message_definition.msgid,
            message_definition.symbol,
            message_definition.old_names,
        )
        if message_definition.old_names:
            number_of_msgid += len(message_definition.old_names)
    assert len(empty_msgid_store) == number_of_msgid


def test_add_msgid_and_symbol(empty_msgid_store: MessageIdStore) -> None:
    empty_msgid_store.add_msgid_and_symbol("E1235", "new-sckiil")
    empty_msgid_store.add_legacy_msgid_and_symbol("C1235", "old-sckiil", "E1235")
    assert len(empty_msgid_store) == 2
    message_ids = empty_msgid_store.get_active_msgids("E1235")
    assert len(message_ids) == 1
    assert message_ids[0] == "E1235"
    message_ids = empty_msgid_store.get_active_msgids("old-sckiil")
    assert len(message_ids) == 1
    assert message_ids[0] == "E1235"
    assert empty_msgid_store.get_symbol("C1235") == "old-sckiil"
    assert empty_msgid_store.get_symbol("E1235") == "new-sckiil"
    assert empty_msgid_store.get_msgid("old-sckiil") == "C1235"
    assert empty_msgid_store.get_msgid("new-sckiil") == "E1235"
    with pytest.raises(UnknownMessageError):
        empty_msgid_store.get_symbol("C1234")
    with pytest.raises(UnknownMessageError):
        empty_msgid_store.get_msgid("not-exist")


def test_duplicate_symbol(empty_msgid_store: MessageIdStore) -> None:
    empty_msgid_store.add_msgid_and_symbol("W1234", "warning-symbol")
    with pytest.raises(InvalidMessageError) as error:
        empty_msgid_store.check_msgid_and_symbol("W1234", "other-symbol")
    assert (
        "Message id 'W1234' cannot have both 'other-symbol' and 'warning-symbol' as symbolic name."
        in str(error.value)
    )


def test_duplicate_msgid(msgid_store: MessageIdStore) -> None:
    msgid_store.add_msgid_and_symbol("W1234", "warning-symbol")
    with pytest.raises(InvalidMessageError) as error:
        msgid_store.check_msgid_and_symbol("W1235", "warning-symbol")
    assert (
        "Message symbol 'warning-symbol' cannot be used for 'W1234' and 'W1235'"
        in str(error.value)
    )


def test_exclusivity_of_msgids() -> None:
    """Test to see if all checkers have an exclusive message id prefix."""
    err_msg = (
        "{} has the same prefix ('{}') as the '{}' checker. Please make sure the prefix "
        "is unique for each checker. You can use 'script/get_unused_message_id_category.py' "
        "to get a unique id."
    )
    runner = Run(["--enable-all-extensions", EMPTY_FILE], exit=False)

    # Some pairs are hard-coded as they are pre-existing and non-exclusive,
    # and we don't want to rename them for backwards compatibility
    checker_id_pairs = {
        "00": ("main", "miscellaneous"),
        "01": (
            "basic",
            "refactoring",
            "consider_ternary_expression",
            "while_used",
            "docstyle",
            "deprecated_builtins",
        ),
        "02": ("classes", "refactoring", "multiple_types"),
        "03": ("classes", "format"),
        "04": ("imports", "spelling"),
        "05": ("consider-using-any-or-all", "miscellaneous"),
        "07": ("exceptions", "broad_try_clause", "overlap-except"),
        "12": ("design", "logging"),
        "17": ("async", "refactoring"),
        "20": ("compare-to-zero", "empty-comment", "magic-value"),
    }

    for msgid, definition in runner.linter.msgs_store._messages_definitions.items():
        if definition.shared:
            continue
        if msgid[1:3] in checker_id_pairs:
            assert (
                definition.checker_name in checker_id_pairs[msgid[1:3]]
            ), err_msg.format(msgid, msgid[1:3], checker_id_pairs[msgid[1:3]][0])
        else:
            checker_id_pairs[msgid[1:3]] = (definition.checker_name,)



"""Regression test for issue 5770: NewType created with f-string
See: https://github.com/pylint-dev/pylint/issues/5770
"""
from typing import NewType

def make_new_type(suffix):
    """Dynamically create a NewType with `suffix`"""
    new_type = NewType(f'IntRange_{suffix}', suffix)
    print(new_type)



"""Tests for singledispatchmethod-function"""
# pylint: disable=missing-class-docstring, missing-function-docstring,too-few-public-methods


from functools import singledispatch, singledispatchmethod


class BoardRight:
    @singledispatch
    @staticmethod
    def convert_position(position):
        pass

    @convert_position.register
    @staticmethod
    def _(position: str) -> tuple:
        position_a, position_b = position.split(",")
        return (int(position_a), int(position_b))

    @convert_position.register
    @staticmethod
    def _(position: tuple) -> str:
        return f"{position[0]},{position[1]}"


class Board:
    @singledispatchmethod  # [singledispatchmethod-function]
    @staticmethod
    def convert_position(position):
        pass

    @convert_position.register  # [singledispatchmethod-function]
    @staticmethod
    def _(position: str) -> tuple:
        position_a, position_b = position.split(",")
        return (int(position_a), int(position_b))

    @convert_position.register  # [singledispatchmethod-function]
    @staticmethod
    def _(position: tuple) -> str:
        return f"{position[0]},{position[1]}"



"""Test that valid class attribute doesn't trigger errors"""
# pylint: disable=missing-docstring,too-few-public-methods


class Clazz:
    "dummy class"

    def __init__(self):
        self.topic = 5
        self._data = 45

    def change_type(self, new_class):
        """Change type"""
        self.__class__ = new_class

    def do_nothing(self):
        "I do nothing useful"
        return self.topic + 56


class Base:
    _class_prop: int


class Child(Base):
    _class_prop = 42

    def method(self):
        print(self._class_prop)


Child().method()



# pylint: disable=missing-docstring
from json import JSONEncoder

class Control: # [too-few-public-methods]
    ...


class MyJsonEncoder(JSONEncoder):
    ...

class InheritedInModule(Control):
    """This class inherits from a class that doesn't have enough methods,
    and its parent is excluded via config, so it doesn't raise."""



# pylint: disable=C0111,R0903,T1234  # [unknown-option-value,use-symbolic-message-instead,use-symbolic-message-instead]
# pylint: enable=c0111,w0223   # [use-symbolic-message-instead,use-symbolic-message-instead]

def my_function(arg):  # [missing-function-docstring]
    return arg or True

# pylint: disable=C0111  # [use-symbolic-message-instead]
# pylint: enable=R0903  # [use-symbolic-message-instead]
# pylint: disable=R0903  # [use-symbolic-message-instead]


def foo():  # pylint: disable=C0102  # [use-symbolic-message-instead]
    return 1


def toto():  # pylint: disable=C0102,R1711  # [use-symbolic-message-instead,use-symbolic-message-instead]
    return


def test_enabled_by_id_msg():  # pylint: enable=C0111  # [use-symbolic-message-instead,missing-function-docstring]
    pass


def baz():  # pylint: disable=blacklisted-name
    return 1



"""Test that invalid module name do not cause errors when using an alias"""
# pylint: disable=import-error, wrong-import-position
import sys
import os

# allow module imports to test that this is indeed a valid python file
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

import non_ascii_name_loł as foobar

print(foobar)



"""Tests for self-defined Enum members (https://github.com/pylint-dev/pylint/issues/5138)"""
# pylint: disable=missing-docstring
from enum import IntEnum, Enum


class Day(IntEnum):
    MONDAY = (1, "Mon")
    TUESDAY = (2, "Tue")
    WEDNESDAY = (3, "Wed")
    THURSDAY = (4, "Thu")
    FRIDAY = (5, "Fri")
    SATURDAY = (6, "Sat")
    SUNDAY = (7, "Sun")

    def __new__(cls, value, abbr=None):
        obj = int.__new__(cls, value)
        obj._value_ = value
        if abbr:
            obj.abbr = abbr
        else:
            obj.abbr = ""
        return obj

    def __repr__(self):
        return f"{self._value_}: {self.foo}"  # [no-member]


print(Day.FRIDAY.abbr)
print(Day.FRIDAY.foo)  # [no-member]


class Length(Enum):
    METRE = "metre", "m"
    MILE = "mile", "m", True

    def __init__(self, text: str, unit: str,  is_imperial: bool = False):
        self.text: str = text
        self.unit: str = unit
        if is_imperial:
            self.suffix = " (imp)"
        else:
            self.suffix = ""


print(f"100 {Length.METRE.unit}{Length.METRE.suffix}")
print(Length.MILE.foo)  # [no-member]


class Binary(int, Enum):
    ZERO = 0
    ONE = 1

    def __init__(self, value: int) -> None:
        super().__init__()
        self.str, self.inverse = str(value), abs(value - 1)

        def no_op(_value):
            pass
        value_squared = value ** 2
        no_op(value_squared)


print(f"1={Binary.ONE.value} (Inverted: {Binary.ONE.inverse}")



'''Assert check example'''

# pylint: disable=comparison-with-itself, comparison-of-constants, line-too-long
assert (1 == 1, 2 == 2), "message is raised even when there is an assert message"  # [assert-on-tuple]
assert (1 == 1, 2 == 2) # [assert-on-tuple]
assert 1 == 1, "no error"
assert (1 == 1, ), "message is raised even when there is an assert message"  # [assert-on-tuple]
assert (1 == 1, ) # [assert-on-tuple]
assert (1 == 1, 2 == 2, 3 == 5), "message is raised even when there is an assert message"  # [assert-on-tuple]
assert ()
assert (True, 'error msg') # [assert-on-tuple]



"""Tests for used-before-assignment false positive from ternary expression with walrus operator"""
# pylint: disable=unnecessary-lambda-assignment, unused-variable, disallowed-name, invalid-name

def invalid():
    """invalid cases that will trigger used-before-assignment"""
    var = foo(a, '', '')  # [used-before-assignment]
    print(str(1 if (a:=-1) else 0))
    var = bar(b)  # [used-before-assignment]
    var = c*c  # [used-before-assignment]
    var = 1 if (b:=-1) else 0
    var = 1 if (c:=-1) else 0

def attribute_call_valid():
    """assignment with attribute calls"""
    var = (a if (a:='a') else '').lower()
    var = ('' if (b:='b') else b).lower()
    var = (c if (c:='c') else c).upper().lower().replace('', '').strip()
    var = ''.strip().replace('', '' + (e if (e:='e') else '').lower())

def function_call_arg_valid():
    """assignment as function call arguments"""
    var = str(a if (a:='a') else '')
    var = str('' if (b:='b') else b)
    var = foo(1, c if (c:=1) else 0, 1)
    print(foo('', '', foo('', str(int(d if (d:='1') else '')), '')))

def function_call_keyword_valid():
    """assignment as function call keywords"""
    var = foo(x=a if (a:='1') else '', y='', z='')
    var = foo(x='', y=foo(x='', y='', z=b if (b:='1') else ''), z='')

def dictionary_items_valid():
    """assignment as dictionary keys/values"""
    var = {
        0: w if (w:=input()) else "",
    }
    var = {
        x if (x:=input()) else "": 0,
    }
    var = {
        0: y if (y:=input()) else "",
        z if (z:=input()) else "": 0,
    }

def complex_valid():
    """assignment within complex call expression"""
    var = str(bar(bar(a if (a:=1) else 0))).lower().upper()
    print(foo(x=foo(''.replace('', str(b if (b:=1) else 0).upper()), '', z=''), y='', z=''))

def foo(x, y, z):
    """helper function for tests"""
    return x+y+z

bar = lambda x : x



"""Tests for missing-yield-doc and missing-yield-type-doc for Sphinx style docstrings"""
# pylint: disable=missing-function-docstring, unused-argument, function-redefined
# pylint: disable=invalid-name, undefined-variable
import typing


# Test missing yields typing docstring
def generator() -> typing.Iterator[int]:
    """A simple function for checking type hints.

    :returns: The number 0
    """
    yield 0



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import os
import sys
from pathlib import Path

import pytest

from pylint.testutils.utils import _test_cwd, _test_environ_pythonpath, _test_sys_path


def test__test_sys_path_no_arg() -> None:
    sys_path = sys.path
    with _test_sys_path():
        assert sys.path == sys_path
        new_sys_path = ["new_sys_path"]
        sys.path = new_sys_path
        assert sys.path == new_sys_path
    assert sys.path == sys_path


def test__test_sys_path() -> None:
    sys_path = sys.path
    new_sys_path = [".", "/home"]
    with _test_sys_path(new_sys_path):
        assert sys.path == new_sys_path
        newer_sys_path = ["new_sys_path"]
        sys.path = newer_sys_path
        assert sys.path == newer_sys_path
    assert sys.path == sys_path


def test__test_cwd_no_arg(tmp_path: Path) -> None:
    cwd = os.getcwd()
    with _test_cwd():
        assert os.getcwd() == cwd
        os.chdir(tmp_path)
        assert os.getcwd() == str(tmp_path)
    assert os.getcwd() == cwd


def test__test_cwd(tmp_path: Path) -> None:
    cwd = os.getcwd()
    with _test_cwd(tmp_path):
        assert os.getcwd() == str(tmp_path)
        new_path = tmp_path / "another_dir"
        new_path.mkdir()
        os.chdir(new_path)
        assert os.getcwd() == str(new_path)
    assert os.getcwd() == cwd


@pytest.mark.parametrize("old_pythonpath", ["./oldpath/:", None])
def test__test_environ_pythonpath_no_arg(old_pythonpath: str) -> None:
    real_pythonpath = os.environ.get("PYTHONPATH")
    with _test_environ_pythonpath(old_pythonpath):
        with _test_environ_pythonpath():
            assert os.environ.get("PYTHONPATH") is None
            new_pythonpath = "./whatever/:"
            os.environ["PYTHONPATH"] = new_pythonpath
            assert os.environ.get("PYTHONPATH") == new_pythonpath
        assert os.environ.get("PYTHONPATH") == old_pythonpath
    assert os.environ.get("PYTHONPATH") == real_pythonpath


@pytest.mark.parametrize("old_pythonpath", ["./oldpath/:", None])
def test__test_environ_pythonpath(old_pythonpath: str) -> None:
    real_pythonpath = os.environ.get("PYTHONPATH")
    with _test_environ_pythonpath(old_pythonpath):
        new_pythonpath = "./whatever/:"
        with _test_environ_pythonpath(new_pythonpath):
            assert os.environ.get("PYTHONPATH") == new_pythonpath
            newer_pythonpath = "./something_else/:"
            os.environ["PYTHONPATH"] = newer_pythonpath
            assert os.environ.get("PYTHONPATH") == newer_pythonpath
        assert os.environ.get("PYTHONPATH") == old_pythonpath
    assert os.environ.get("PYTHONPATH") == real_pythonpath



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Tests for the pylint.checkers.utils module."""

from __future__ import annotations

import astroid
import pytest
from astroid import nodes

from pylint.checkers import utils


@pytest.mark.parametrize(
    "name,expected",
    [
        ("min", True),
        ("__builtins__", True),
        ("__path__", False),
        ("__file__", False),
        ("whatever", False),
        ("mybuiltin", False),
    ],
)
def testIsBuiltin(name: str, expected: bool) -> None:
    assert utils.is_builtin(name) == expected


@pytest.mark.parametrize(
    "fn,kw",
    [("foo(3)", {"keyword": "bar"}), ("foo(one=a, two=b, three=c)", {"position": 1})],
)
def testGetArgumentFromCallError(fn: str, kw: dict[str, int] | dict[str, str]) -> None:
    with pytest.raises(utils.NoSuchArgumentError):
        node = astroid.extract_node(fn)
        utils.get_argument_from_call(node, **kw)


@pytest.mark.parametrize(
    "fn,kw", [("foo(bar=3)", {"keyword": "bar"}), ("foo(a, b, c)", {"position": 1})]
)
def testGetArgumentFromCallExists(fn: str, kw: dict[str, int] | dict[str, str]) -> None:
    node = astroid.extract_node(fn)
    assert utils.get_argument_from_call(node, **kw) is not None


def testGetArgumentFromCall() -> None:
    node = astroid.extract_node("foo(a, not_this_one=1, this_one=2)")
    arg = utils.get_argument_from_call(node, position=2, keyword="this_one")
    assert arg.value == 2

    node = astroid.extract_node("foo(a)")
    with pytest.raises(utils.NoSuchArgumentError):
        utils.get_argument_from_call(node, position=1)
    with pytest.raises(ValueError):
        utils.get_argument_from_call(node, None, None)
    name = utils.get_argument_from_call(node, position=0)
    assert name.name == "a"


def test_error_of_type() -> None:
    code = astroid.extract_node(
        """
    try: pass
    except AttributeError: #@
         pass
    try: pass
    except Exception: #@
         pass
    except: #@
         pass
    """
    )
    assert utils.error_of_type(code[0], AttributeError)
    assert utils.error_of_type(code[0], (AttributeError,))
    assert not utils.error_of_type(code[0], Exception)
    assert utils.error_of_type(code[1], Exception)


def test_node_ignores_exception() -> None:
    code = astroid.extract_node(
        """
    try:
        1/0 #@
    except ZeroDivisionError:
        pass
    try:
        1/0 #@
    except Exception:
        pass
    try:
        2/0 #@
    except:
        pass
    try:
        1/0 #@
    except ValueError:
        pass
    """
    )
    assert utils.node_ignores_exception(code[0], ZeroDivisionError)
    assert not utils.node_ignores_exception(code[1], ZeroDivisionError)
    assert not utils.node_ignores_exception(code[2], ZeroDivisionError)
    assert not utils.node_ignores_exception(code[3], ZeroDivisionError)


def test_is_subclass_of_node_b_derived_from_node_a() -> None:
    code = astroid.extract_node(
        """
    class Superclass: #@
        pass

    class Subclass(Superclass): #@
        pass
    """
    )
    assert utils.is_subclass_of(code[1], code[0])


def test_is_subclass_of_node_b_not_derived_from_node_a() -> None:
    code = astroid.extract_node(
        """
    class OneClass: #@
        pass

    class AnotherClass: #@
        pass
    """
    )
    assert not utils.is_subclass_of(code[1], code[0])


def test_is_subclass_of_not_classdefs() -> None:
    node = astroid.extract_node(
        """
    class OneClass: #@
        pass
    """
    )
    assert not utils.is_subclass_of(None, node)
    assert not utils.is_subclass_of(node, None)
    assert not utils.is_subclass_of(None, None)


def test_parse_format_method_string() -> None:
    samples = [
        ("{}", 1),
        ("{}:{}", 2),
        ("{field}", 1),
        ("{:5}", 1),
        ("{:10}", 1),
        ("{field:10}", 1),
        ("{field:10}{{}}", 1),
        ("{:5}{!r:10}", 2),
        ("{:5}{}{{}}{}", 3),
        ("{0}{1}{0}", 2),
        ("Coordinates: {latitude}, {longitude}", 2),
        ("X: {0[0]};  Y: {0[1]}", 1),
        ("{:*^30}", 1),
        ("{!r:}", 1),
    ]
    for fmt, count in samples:
        keys, num_args, pos_args = utils.parse_format_method_string(fmt)
        keyword_args = len({k for k, _ in keys if not isinstance(k, int)})
        assert keyword_args + num_args + pos_args == count


def test_inherit_from_std_ex_recursive_definition() -> None:
    node = astroid.extract_node(
        """
      import datetime
      class First(datetime.datetime):
        pass
      class Second(datetime.datetime): #@
        pass
      datetime.datetime = First
      datetime.datetime = Second
      """
    )
    assert not utils.inherit_from_std_ex(node)


def test_get_node_last_lineno_simple() -> None:
    node = astroid.extract_node(
        """
        pass
    """
    )
    assert utils.get_node_last_lineno(node) == 2


def test_get_node_last_lineno_if_simple() -> None:
    node = astroid.extract_node(
        """
        if True:
            print(1)
            pass
        """
    )
    assert utils.get_node_last_lineno(node) == 4


def test_get_node_last_lineno_if_elseif_else() -> None:
    node = astroid.extract_node(
        """
        if True:
            print(1)
        elif False:
            print(2)
        else:
            print(3)
        """
    )
    assert utils.get_node_last_lineno(node) == 7


def test_get_node_last_lineno_while() -> None:
    node = astroid.extract_node(
        """
        while True:
            print(1)
        """
    )
    assert utils.get_node_last_lineno(node) == 3


def test_get_node_last_lineno_while_else() -> None:
    node = astroid.extract_node(
        """
        while True:
            print(1)
        else:
            print(2)
        """
    )
    assert utils.get_node_last_lineno(node) == 5


def test_get_node_last_lineno_for() -> None:
    node = astroid.extract_node(
        """
        for x in range(0, 5):
            print(1)
        """
    )
    assert utils.get_node_last_lineno(node) == 3


def test_get_node_last_lineno_for_else() -> None:
    node = astroid.extract_node(
        """
        for x in range(0, 5):
            print(1)
        else:
            print(2)
        """
    )
    assert utils.get_node_last_lineno(node) == 5


def test_get_node_last_lineno_try() -> None:
    node = astroid.extract_node(
        """
        try:
            print(1)
        except ValueError:
            print(2)
        except Exception:
            print(3)
        """
    )
    assert utils.get_node_last_lineno(node) == 7


def test_get_node_last_lineno_try_except_else() -> None:
    node = astroid.extract_node(
        """
        try:
            print(1)
        except Exception:
            print(2)
            print(3)
        else:
            print(4)
        """
    )
    assert utils.get_node_last_lineno(node) == 8


def test_get_node_last_lineno_try_except_finally() -> None:
    node = astroid.extract_node(
        """
        try:
            print(1)
        except Exception:
            print(2)
        finally:
            print(4)
        """
    )
    assert utils.get_node_last_lineno(node) == 7


def test_get_node_last_lineno_try_except_else_finally() -> None:
    node = astroid.extract_node(
        """
        try:
            print(1)
        except Exception:
            print(2)
        else:
            print(3)
        finally:
            print(4)
        """
    )
    assert utils.get_node_last_lineno(node) == 9


def test_get_node_last_lineno_with() -> None:
    node = astroid.extract_node(
        """
        with x as y:
            print(1)
            pass
        """
    )
    assert utils.get_node_last_lineno(node) == 4


def test_get_node_last_lineno_method() -> None:
    node = astroid.extract_node(
        """
        def x(a, b):
            print(a, b)
            pass
        """
    )
    assert utils.get_node_last_lineno(node) == 4


def test_get_node_last_lineno_decorator() -> None:
    node = astroid.extract_node(
        """
        @decor()
        def x(a, b):
            print(a, b)
            pass
        """
    )
    assert utils.get_node_last_lineno(node) == 5


def test_get_node_last_lineno_class() -> None:
    node = astroid.extract_node(
        """
        class C(object):
            CONST = True

            def x(self, b):
                print(b)

            def y(self):
                pass
                pass
        """
    )
    assert utils.get_node_last_lineno(node) == 10


def test_get_node_last_lineno_combined() -> None:
    node = astroid.extract_node(
        """
        class C(object):
            CONST = True

            def y(self):
                try:
                    pass
                except:
                    pass
                finally:
                    pass
        """
    )
    assert utils.get_node_last_lineno(node) == 11


def test_if_sys_guard() -> None:
    code = astroid.extract_node(
        """
    import sys
    if sys.version_info > (3, 8):  #@
        pass

    if sys.version_info[:2] > (3, 8):  #@
        pass

    if sys.some_other_function > (3, 8):  #@
        pass
    """
    )
    assert isinstance(code, list) and len(code) == 3

    assert isinstance(code[0], nodes.If)
    assert utils.is_sys_guard(code[0]) is True
    assert isinstance(code[1], nodes.If)
    assert utils.is_sys_guard(code[1]) is True

    assert isinstance(code[2], nodes.If)
    assert utils.is_sys_guard(code[2]) is False


def test_if_typing_guard() -> None:
    code = astroid.extract_node(
        """
    import typing
    import typing as t
    from typing import TYPE_CHECKING

    if typing.TYPE_CHECKING:
        pass  #@

    if t.TYPE_CHECKING:
        pass #@

    if TYPE_CHECKING:
        pass #@

    if typing.SOME_OTHER_CONST:
        pass  #@
    """
    )
    assert isinstance(code, list) and len(code) == 4

    assert isinstance(code[0], nodes.Pass)
    assert utils.in_type_checking_block(code[0]) is True
    assert isinstance(code[1], nodes.Pass)
    assert utils.in_type_checking_block(code[1]) is True
    assert isinstance(code[2], nodes.Pass)
    assert utils.in_type_checking_block(code[2]) is True

    assert isinstance(code[3], nodes.Pass)
    assert utils.in_type_checking_block(code[3]) is False


def test_in_type_checking_block() -> None:
    code = astroid.extract_node(
        """
    if TYPE_CHECKING:  # don't import this!
        import math  #@
    """
    )
    assert utils.in_type_checking_block(code) is False


def test_is_empty_literal() -> None:
    list_node = astroid.extract_node("a = []")
    assert utils.is_base_container(list_node.value)
    not_empty_list_node = astroid.extract_node("a = [1,2,3]")
    assert not utils.is_base_container(not_empty_list_node.value)

    tuple_node = astroid.extract_node("a = ()")
    assert utils.is_base_container(tuple_node.value)
    not_empty_tuple_node = astroid.extract_node("a = (1,2)")
    assert not utils.is_base_container(not_empty_tuple_node.value)

    dict_node = astroid.extract_node("a = {}")
    assert utils.is_empty_dict_literal(dict_node.value)
    not_empty_dict_node = astroid.extract_node("a = {1:1}")
    assert not utils.is_empty_dict_literal(not_empty_dict_node.value)

    string_node = astroid.extract_node("a = ''")
    assert utils.is_empty_str_literal(string_node.value)
    not_empty_string_node = astroid.extract_node("a = 'hello'")
    assert not utils.is_empty_str_literal(not_empty_string_node.value)


def test_is_typing_member() -> None:
    code = astroid.extract_node(
        """
    from typing import Literal as Lit, Set as Literal
    import typing as t

    Literal #@
    Lit #@
    t.Literal #@
    """
    )

    assert not utils.is_typing_member(code[0], ("Literal",))
    assert utils.is_typing_member(code[1], ("Literal",))
    assert utils.is_typing_member(code[2], ("Literal",))

    code = astroid.extract_node(
        """
    Literal #@
    typing.Literal #@
    """
    )
    assert not utils.is_typing_member(code[0], ("Literal",))
    assert not utils.is_typing_member(code[1], ("Literal",))



"""Latest version of astroid (2.6.3) causes RuntimeError:
generator raised StopIteration #4723"""
# pylint: disable=invalid-name,missing-docstring,too-few-public-methods

import contextlib


class A:
    @contextlib.contextmanager
    def get(self):
        yield self


class B(A):
    def play():  # [no-method-argument]
        pass


def func():
    with B().get() as b:
        b.play()  # [too-many-function-args]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

# pylint: disable=redefined-outer-name

from __future__ import annotations

import itertools
from collections.abc import Callable
from pathlib import Path
from typing import cast

import astroid
import pytest
from astroid import AstroidBuildingError, nodes

import pylint.checkers.unicode
import pylint.interfaces
import pylint.testutils

from . import CODEC_AND_MSG, FakeNode


@pytest.fixture()
def bad_char_file_generator(tmp_path: Path) -> Callable[[str, bool, str], Path]:
    """Generates a test file for bad chars.

    The generator also ensures that file generated is correct
    """

    def encode_without_bom(string: str, encoding: str) -> bytes:
        return pylint.checkers.unicode._encode_without_bom(string, encoding)

    # All lines contain a not extra checked invalid character
    lines = (
        "# Example File containing bad ASCII",
        "# invalid char backspace: \b",
        "# Bad carriage-return \r # not at the end",
        "# Invalid char sub: \x1A",
        "# Invalid char esc: \x1B",
    )

    def _bad_char_file_generator(
        codec: str, add_invalid_bytes: bool, line_ending: str
    ) -> Path:
        byte_suffix = b""
        if add_invalid_bytes:
            if codec == "utf-8":
                byte_suffix = b"BAD:\x80abc"
            elif codec == "utf-16":
                byte_suffix = b"BAD:\n"  # Generates Truncated Data
            else:
                byte_suffix = b"BAD:\xc3\x28 "
            byte_suffix = encode_without_bom(" foobar ", codec) + byte_suffix

        line_ending_encoded = encode_without_bom(line_ending, codec)

        # Start content with BOM / codec definition and two empty lines
        content = f"# coding: {codec} \n # \n ".encode(codec)

        # Generate context with the given codec and line ending
        for lineno, line in enumerate(lines):
            byte_line = encode_without_bom(line, codec)
            byte_line += byte_suffix + line_ending_encoded
            content += byte_line

            # Directly test the generated content
            if not add_invalid_bytes:
                # Test that the content is correct and gives no errors
                try:
                    byte_line.decode(codec, "strict")
                except UnicodeDecodeError as e:
                    raise ValueError(
                        f"Line {lineno} did raise unexpected error: {byte_line!r}\n{e}"
                    ) from e
            else:
                try:
                    # But if there was a byte_suffix we expect an error
                    # because that is what we want to test for
                    byte_line.decode(codec, "strict")
                except UnicodeDecodeError:
                    ...
                else:
                    raise ValueError(
                        f"Line {lineno} did not raise decode error: {byte_line!r}"
                    )

        file = tmp_path / "bad_chars.py"
        file.write_bytes(content)
        return file

    return _bad_char_file_generator


class TestBadCharsChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint.checkers.unicode.UnicodeChecker

    checker: pylint.checkers.unicode.UnicodeChecker

    @pytest.mark.parametrize(
        "codec_and_msg, line_ending, add_invalid_bytes",
        [
            pytest.param(
                codec_and_msg,
                line_ending[0],
                suffix[0],
                id=f"{codec_and_msg[0]}_{line_ending[1]}_{suffix[1]}",
            )
            for codec_and_msg, line_ending, suffix in itertools.product(
                CODEC_AND_MSG,
                (("\n", "linux"), ("\r\n", "windows")),
                ((False, "valid_line"), (True, "not_decode_able_line")),
            )
            # Only utf8 can drop invalid lines
            if codec_and_msg[0].startswith("utf") or not suffix[0]
        ],
    )
    def test_find_bad_chars(
        self,
        bad_char_file_generator: Callable[[str, bool, str], Path],
        codec_and_msg: tuple[str, tuple[pylint.testutils.MessageTest]],
        line_ending: str,
        add_invalid_bytes: bool,
    ) -> None:
        """All combinations of bad characters that are accepted by Python at the moment
        are tested in all possible combinations of
          - line ending
          - encoding
          - including not encode-able byte (or not)
        """
        codec, start_msg = codec_and_msg

        start_lines = 2

        file = bad_char_file_generator(codec, add_invalid_bytes, line_ending)

        try:
            # We need to use ast from file as only this function reads bytes and not
            # string
            module = astroid.MANAGER.ast_from_string(file)
        except AstroidBuildingError:
            # pylint: disable-next=redefined-variable-type
            module = cast(nodes.Module, FakeNode(file.read_bytes()))

        expected = [
            *start_msg,
            pylint.testutils.MessageTest(
                msg_id="invalid-character-backspace",
                line=2 + start_lines,
                end_line=2 + start_lines,
                # node=module,
                args=None,
                confidence=pylint.interfaces.HIGH,
                col_offset=27,
                end_col_offset=28,
            ),
            pylint.testutils.MessageTest(
                msg_id="invalid-character-carriage-return",
                line=3 + start_lines,
                end_line=3 + start_lines,
                # node=module,
                args=None,
                confidence=pylint.interfaces.HIGH,
                col_offset=23,
                end_col_offset=24,
            ),
            pylint.testutils.MessageTest(
                msg_id="invalid-character-sub",
                line=4 + start_lines,
                end_line=4 + start_lines,
                # node=module,
                args=None,
                confidence=pylint.interfaces.HIGH,
                col_offset=21,
                end_col_offset=22,
            ),
            pylint.testutils.MessageTest(
                msg_id="invalid-character-esc",
                line=5 + start_lines,
                end_line=5 + start_lines,
                # node=module,
                args=None,
                confidence=pylint.interfaces.HIGH,
                col_offset=21,
                end_col_offset=22,
            ),
        ]
        with self.assertAddsMessages(*expected):
            self.checker.process_module(module)

    @pytest.mark.parametrize(
        "codec_and_msg, char, msg_id",
        [
            pytest.param(
                codec_and_msg,
                char_msg[0],
                char_msg[1],
                id=f"{char_msg[1]}_{codec_and_msg[0]}",
            )
            for codec_and_msg, char_msg in itertools.product(
                CODEC_AND_MSG,
                (
                    ("\0", "invalid-character-nul"),
                    ("\N{ZERO WIDTH SPACE}", "invalid-character-zero-width-space"),
                ),
            )
            # Only utf contains zero width space
            if (
                char_msg[0] != "\N{ZERO WIDTH SPACE}"
                or codec_and_msg[0].startswith("utf")
            )
        ],
    )
    def test_bad_chars_that_would_currently_crash_python(
        self,
        char: str,
        msg_id: str,
        codec_and_msg: tuple[str, tuple[pylint.testutils.MessageTest]],
    ) -> None:
        """Special test for a file containing chars that lead to
        Python or Astroid crashes (which causes Pylint to exit early)
        """
        codec, start_msg = codec_and_msg
        # Create file that will fail loading in astroid.
        # We still want to check this, in case this behavior changes
        content = f"# # coding: {codec}\n# file containing {char} <-\n"
        module = FakeNode(content.encode(codec))

        expected = [
            *start_msg,
            pylint.testutils.MessageTest(
                msg_id=msg_id,
                line=2,
                end_line=2,
                # node=module,
                args=None,
                confidence=pylint.interfaces.HIGH,
                col_offset=19,
                end_col_offset=20,
            ),
        ]

        with self.assertAddsMessages(*expected):
            self.checker.process_module(cast(nodes.Module, module))

    @pytest.mark.parametrize(
        "char, msg, codec",
        [
            pytest.param(
                char.unescaped,
                char.human_code(),
                codec_and_msg[0],
                id=f"{char.name}_{codec_and_msg[0]}",
            )
            for char, codec_and_msg in itertools.product(
                pylint.checkers.unicode.BAD_CHARS, CODEC_AND_MSG
            )
            # Only utf contains zero width space
            if (
                char.unescaped != "\N{ZERO WIDTH SPACE}"
                or codec_and_msg[0].startswith("utf")
            )
        ],
    )
    def test___check_invalid_chars(self, char: str, msg: str, codec: str) -> None:
        """Check function should deliver correct column no matter which codec we used."""
        with self.assertAddsMessages(
            pylint.testutils.MessageTest(
                msg_id=msg,
                line=55,
                args=None,
                confidence=pylint.interfaces.HIGH,
                end_line=55,
                col_offset=5,
                end_col_offset=6,
            )
        ):
            self.checker._check_invalid_chars(f"#234{char}".encode(codec), 55, codec)



"""Tests for missing-raises-doc and missing-raises-type-doc"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, import-error, unused-variable, no-member, try-except-raise
import collections

from fake_package import BadError
from unknown import Unknown


def test_ignores_no_docstring(self):
    raise RuntimeError("hi")


def test_ignores_unknown_style(self):
    """This is a docstring."""
    raise RuntimeError("hi")


def test_ignores_raise_uninferable(self):
    """This is a docstring.

    :raises NameError: Never
    """
    raise Unknown("hi")
    raise NameError("hi")  # [unreachable]


def test_ignores_returns_from_inner_functions(self):  # [missing-raises-doc]
    """This is a docstring.
    We do NOT expect a warning about the OSError in inner_func!

    :raises NameError: Never
    """

    def ex_func(val):
        def inner_func(value):
            return OSError(value)

        return RuntimeError(val)

    raise ex_func("hi")
    raise NameError("hi")  # [unreachable]


def test_ignores_returns_use_only_names():
    """This is a docstring

    :raises NameError: Never
    """

    def inner_func():
        return 42

    raise inner_func()  # [raising-bad-type]


def test_ignores_returns_use_only_exception_instances():
    """This is a docstring

    :raises MyException: Never
    """

    class MyException(Exception):
        """A docstring"""

    def inner_func():
        return MyException

    raise inner_func()


def test_no_crash_when_inferring_handlers():
    """raises

    :raise U: pass
    """
    try:
        pass
    except collections.U as exc:
        raise


def test_no_crash_when_cant_find_exception():
    """raises

    :raise U: pass
    """
    try:
        pass
    except U as exc:
        raise


def test_no_error_notimplemented_documented():
    """
    Raises:
        NotImplementedError: When called.
    """
    raise NotImplementedError


def test_finds_short_name_exception():
    """Do something.

    Raises:
        ~fake_package.exceptions.BadError: When something bad happened.
    """
    raise BadError("A bad thing happened.")



# pylint: disable=missing-docstring,no-self-argument


class Cls:
    def __init__(self, obj, /):
        self.obj = obj

    # regression tests for no-method-argument getting reported
    # instead of no-self-argument
    def varargs(*args):
        """A method without a self argument but with *args."""

    def kwargs(**kwargs):
        """A method without a self argument but with **kwargs."""

    def varargs_and_kwargs(*args, **kwargs):
        """A method without a self argument but with *args and **kwargs."""



# pylint: disable=missing-docstring
from collections import namedtuple

NoteHash = namedtuple('NoteHash', ['Pitch', 'Duration', 'Offset'])  # [prefer-typing-namedtuple]

class SearchMatch(
    namedtuple('SearchMatch', ['els', 'index', 'iterator'])  # [prefer-typing-namedtuple]
):
    """Adapted from primer package `music21`."""



"""Emit a message for iteration through dict keys and subscripting dict with key."""
# pylint: disable=line-too-long,missing-docstring,unsubscriptable-object,too-few-public-methods,redefined-outer-name,use-dict-literal,modified-iterating-dict

def bad():
    a_dict = {1: 1, 2: 2, 3: 3}
    for k in a_dict:  # [consider-using-dict-items]
        print(a_dict[k])
    another_dict = dict()
    for k in another_dict:  # [consider-using-dict-items]
        print(another_dict[k])


def good():
    a_dict = {1: 1, 2: 2, 3: 3}
    for k in a_dict:
        print(k)

out_of_scope_dict = dict()

def another_bad():
    for k in out_of_scope_dict:  # [consider-using-dict-items]
        print(out_of_scope_dict[k])

def another_good():
    for k in out_of_scope_dict:
        k = 1
        k = 2
        k = 3
        print(out_of_scope_dict[k])


b_dict = {}
for k2 in b_dict:  # Should not emit warning, key access necessary
    b_dict[k2] = 2

for k2 in b_dict:  # Should not emit warning, key access necessary (AugAssign)
    b_dict[k2] += 2

# Warning should be emitted in this case
for k6 in b_dict:  # [consider-using-dict-items]
    val = b_dict[k6]
    b_dict[k6] = 2

for k3 in b_dict:  # [consider-using-dict-items]
    val = b_dict[k3]

for k4 in b_dict.keys():  # [consider-iterating-dictionary,consider-using-dict-items]
    val = b_dict[k4]

class Foo:
    c_dict = {}

# Should emit warning when iterating over a dict attribute of a class
for k5 in Foo.c_dict:  # [consider-using-dict-items]
    val = Foo.c_dict[k5]

c_dict = {}

# Should NOT emit warning whey key used to access a different dict
for k5 in Foo.c_dict:  # This is fine
    val = b_dict[k5]

for k5 in Foo.c_dict:  # This is fine
    val = c_dict[k5]

# Should emit warning within a list/dict comprehension
val = {k9: b_dict[k9] for k9 in b_dict}  # [consider-using-dict-items]
val = [(k7, b_dict[k7]) for k7 in b_dict]  # [consider-using-dict-items]

# Should emit warning even when using dict attribute of a class within comprehension
val = [(k7, Foo.c_dict[k7]) for k7 in Foo.c_dict]  # [consider-using-dict-items]
val = any(True for k8 in Foo.c_dict if Foo.c_dict[k8])  # [consider-using-dict-items]

# Should emit warning when dict access done in ``if`` portion of comprehension
val = any(True for k8 in b_dict if b_dict[k8])  # [consider-using-dict-items]

# Should NOT emit warning whey key used to access a different dict
val = [(k7, b_dict[k7]) for k7 in Foo.c_dict]
val = any(True for k8 in Foo.c_dict if b_dict[k8])

# Should NOT emit warning, essentially same check as above
val = [(k7, c_dict[k7]) for k7 in Foo.c_dict]
val = any(True for k8 in Foo.c_dict if c_dict[k8])

# Should emit warning, using .keys() of Foo.c_dict
val = any(True for k8 in Foo.c_dict.keys() if Foo.c_dict[k8])  # [consider-iterating-dictionary,consider-using-dict-items]

# Test false positive described in #4630
# (https://github.com/pylint-dev/pylint/issues/4630)

d = {'key': 'value'}

for k in d:  # this is fine, with the reassignment of d[k], d[k] is necessary
    d[k] += '123'
    if '1' in d[k]:  # index lookup necessary here, do not emit error
        print('found 1')

for k in d:  # if this gets rewritten to d.items(), we are back to the above problem
    d[k] = d[k] + 1
    if '1' in d[k]:  # index lookup necessary here, do not emit error
        print('found 1')

for k in d:  # [consider-using-dict-items]
    if '1' in d[k]:  # index lookup necessary here, do not emit error
        print('found 1')



"""Check possible unbalanced dict unpacking """
# pylint: disable=missing-function-docstring, invalid-name
# pylint: disable=unused-variable, redefined-outer-name, line-too-long

def dict_vals():
    a, b, c, d, e, f, g = {1: 2}.values()  # [unbalanced-dict-unpacking]
    return a, b

def dict_keys():
    a, b, c, d, e, f, g = {1: 2, "hi": 20}.keys()  # [unbalanced-dict-unpacking]
    return a, b


def dict_items():
    tupe_one, tuple_two = {1: 2, "boo": 3}.items()
    tupe_one, tuple_two, tuple_three = {1: 2, "boo": 3}.items()  # [unbalanced-dict-unpacking]
    return tuple_three

def all_dict():
    a, b, c, d, e, f, g = {1: 2, 3: 4}  # [unbalanced-dict-unpacking]
    return a

for a, b, c, d, e, f, g in {1: 2}.items():  # [unbalanced-dict-unpacking]
    pass

# This is an instance of dict-items-missing-iter, so unbalanced-dict-unpacking is suppressed
for key, value in {1: 2}:
    pass

for key, value in {1: 2}.keys():  # [unbalanced-dict-unpacking, consider-iterating-dictionary]
    pass

for key, value in {1: 2}.values():  # [unbalanced-dict-unpacking]
    pass

empty = {}

# this should not raise unbalanced-dict because it is valid code using `items()`
for key, value in empty.items():
    print(key)
    print(value)

for key, val in {1: 2}.items():
    print(key)

populated = {2: 1}
for key, val in populated.items():
    print(key)

key, val = populated.items()  # [unbalanced-dict-unpacking]

for key, val in {1: 2, 3: 4, 5: 6}.items():
    print(key)

key, val = {1: 2, 3: 4, 5: 6}.items()  # [unbalanced-dict-unpacking]

a, b, c = {}  # [unbalanced-dict-unpacking]

for k in {'key': 'value', 1: 2}.items():
    print(k)

for k, _ in {'key': 'value'}.items():
    print(k)

for _, _ in {'key': 'value'}.items():
    print(_)

for _, val in {'key': 'value'}.values():  # [unbalanced-dict-unpacking]
    print(val)

for key, *val in {'key': 'value', 1: 2}.items():
    print(key)

for *key, val in {'key': 'value', 1: 2}.items():
    print(key)


for key, *val in {'key': 'value', 1: 2, 20: 21}.values():  # [unbalanced-dict-unpacking]
    print(key)

for *key, val in {'key': 'value', 1: 2, 20: 21}.values():  # [unbalanced-dict-unpacking]
    print(key)

one, *others = {1: 2, 3: 4, 5: 6}.items()
one, *others, last = {1: 2, 3: 4, 5: 6}.items()

one, *others = {1: 2, 3: 4, 5: 6}.values()
one, *others, last = {1: 2, 3: 4, 5: 6}.values()

_, *others = {1: 2, 3: 4, 5: 6}.items()
_, *others = {1: 2, 3: 4, 5: 6}.values()
_, others = {1: 2, 3: 4, 5: 6}.values()  # [unbalanced-dict-unpacking]


for value1, value2 in {1: 2, 3: 4}.values():  # [unbalanced-dict-unpacking]
    pass

for value1, value2 in {1: (2, 3), 4: 5}.values():  # [unbalanced-dict-unpacking]
    pass

for value1, value2 in {1: 2, 3: (4, 5)}.values():  # [unbalanced-dict-unpacking]
    pass

for value1, value2 in {1: (2, 3, 4), 5: (6, 7)}.values():  # [unbalanced-dict-unpacking]
    pass

for value1, value2, value3 in {1: (2, 3, 4), 5: (6, 7)}.values():  # [unbalanced-dict-unpacking]
    pass

# These should not raise unbalanced-dict since the there are the same number of targets and values in every expression
for value1, value2 in {1: (2, 3), 4: (5, 6)}.values():
    pass

for value1, value2, value3 in {1: (2, 3, 4), 5: (6, 7, 8)}.values():
    pass

for value1, value2 in {1: [2, 3], 4: {5, 6}}.values():
    pass

for value1, value2 in {1: {2: 3, 4: 5}, 6: {7: 8, 9: 10}}.values():
    pass

for value1, value2 in {1: [1, 2, 3][0:2]}.values():
    pass



"""Test generic alias support with mix of typing.py and stdlib types (PY39+)."""
# flake8: noqa
# pylint: disable=missing-docstring,pointless-statement
# pylint: disable=too-few-public-methods,multiple-statements,line-too-long
import collections
import collections.abc
import contextlib
import re
import typing

# Type annotations
var_orderedDict: collections.OrderedDict[int, str]
var_container: collections.abc.Container[int]
var_sequence: collections.abc.Sequence[int]
var_iterable: collections.abc.Iterable[int]
var_awaitable: collections.abc.Awaitable[int]
var_pattern: re.Pattern[int]
var_bytestring: collections.abc.ByteString
var_hashable: collections.abc.Hashable
var_ContextManager: contextlib.AbstractContextManager[int]


# No implementation required for 'builtins'
class DerivedListIterable(typing.List[typing.Iterable[int]]):
    pass


# Missing implementation for 'collections.abc' derived classes
class DerivedHashable(typing.Hashable):  # [abstract-method]  # __hash__
    pass

class DerivedIterable(typing.Iterable[int]):  # [abstract-method]  # __iter__
    pass

class DerivedCollection(typing.Collection[int]):  # [abstract-method,abstract-method,abstract-method]  # __contains__, __iter__, __len__
    pass



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import itertools
import unicodedata
from pathlib import Path
from typing import cast

import astroid
import pytest
from astroid import nodes

import pylint.checkers.unicode
import pylint.interfaces
import pylint.testutils

from . import FakeNode

UNICODE_TESTS = Path(__file__).parent.parent.parent / "regrtest_data" / "unicode"


class TestBidirectionalUnicodeChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint.checkers.unicode.UnicodeChecker

    checker: pylint.checkers.unicode.UnicodeChecker

    def test_finds_bidirectional_unicode_that_currently_not_parsed(self) -> None:
        """Test an example from https://github.com/nickboucher/trojan-source/tree/main/Python
        that is currently not working Python but producing a syntax error

        So we test this to make sure it stays like this
        """

        test_file = UNICODE_TESTS / "invisible_function.txt"

        with pytest.raises(astroid.AstroidSyntaxError):
            astroid.MANAGER.ast_from_string(test_file.read_text("utf-8"))

        with pytest.raises(AssertionError):
            # The following errors are not risen at the moment,
            # But we keep this in order to allow writing the test fast, if
            # the condition above isn't met anymore.
            module = FakeNode(test_file.read_bytes())
            with self.assertAddsMessages(
                pylint.testutils.MessageTest(
                    msg_id="bidirectional-unicode",
                    confidence=pylint.interfaces.HIGH,
                    # node=module,
                    line=6,
                    end_line=10,
                    col_offset=0,
                    end_col_offset=17,
                ),
                pylint.testutils.MessageTest(
                    msg_id="bidirectional-unicode",
                    confidence=pylint.interfaces.HIGH,
                    line=10,
                    # node=module,
                    end_line=10,
                    col_offset=0,
                    end_col_offset=20,
                ),
            ):
                self.checker.process_module(cast(nodes.Module, module))

    @pytest.mark.parametrize(
        "bad_string, codec",
        [
            pytest.param(
                char,
                codec,
                id=f"{unicodedata.name(char)}_{codec}".replace(" ", "_"),
            )
            for char, codec in itertools.product(
                pylint.checkers.unicode.BIDI_UNICODE,
                ("utf-8", "utf-16le", "utf-16be", "utf-32le", "utf-32be"),
            )
        ],
    )
    def test_find_bidi_string(self, bad_string: str, codec: str) -> None:
        """Ensure that all Bidirectional strings are detected.

        Tests also UTF-16 and UTF-32.
        """
        expected = pylint.testutils.MessageTest(
            msg_id="bidirectional-unicode",
            confidence=pylint.interfaces.HIGH,
            line=1,
            # node=module,
            end_line=1,
            col_offset=0,
            end_col_offset=3,
        )

        with self.assertAddsMessages(expected):
            self.checker._check_bidi_chars(f"# {bad_string}".encode(codec), 1, codec)



"""Test if pylint sees names inside string literal type annotations. #3299"""
# pylint: disable=too-few-public-methods

from argparse import ArgumentParser, Namespace
import os
from os import PathLike
from pathlib import Path
from typing import NoReturn, Set

# unused-import shouldn't be emitted for Path
example1: Set["Path"] = set()

def example2(_: "ArgumentParser") -> "NoReturn":
    """unused-import shouldn't be emitted for ArgumentParser or NoReturn."""
    while True:
        pass

def example3(_: "os.PathLike[str]") -> None:
    """unused-import shouldn't be emitted for os."""

def example4(_: "PathLike[str]") -> None:
    """unused-import shouldn't be emitted for PathLike."""

# pylint shouldn't crash with the following strings in a type annotation context
example5: Set[""]
example6: Set[" "]
example7: Set["?"]

class Class:
    """unused-import shouldn't be emitted for Namespace"""
    cls: "Namespace"



"""Checks for redundant Union typehints in assignments"""
# pylint: disable=deprecated-typing-alias,consider-alternative-union-syntax,consider-using-alias,invalid-name,unused-argument,missing-function-docstring

from __future__ import annotations
from typing import Union, Optional, Sequence

#  +1: [redundant-typehint-argument, redundant-typehint-argument]
ANSWER_0: Union[int, int, str, bool, float, str] = 0
ANSWER_1: Optional[int] = 1
ANSWER_2: Sequence[int] = [2]
ANSWER_3: Union[list[int], str, int, bool, list[int]] = 3  # [redundant-typehint-argument]
ANSWER_4: Optional[None] = None   # [redundant-typehint-argument]
ANSWER_5: Optional[list[int]] = None
ANSWER_6: Union[None, None] = None   # [redundant-typehint-argument]
#  +1: [redundant-typehint-argument]
ANSWER_7: Union[list[int], dict[int], dict[list[int]], list[str], list[str]] = [7]
ANSWER_8: int | int = 8  # [redundant-typehint-argument]
ANSWER_9: str | int | None | int | bool = 9   # [redundant-typehint-argument]
ANSWER_10: dict | list[int] | float | str | int | bool = 10
#  +1: [redundant-typehint-argument]
ANSWER_11: list[int] | dict[int] | dict[list[int]] | list[str] | list[str] = ['string']

# Multiple warnings for the same repeated type
#  +1: [redundant-typehint-argument, redundant-typehint-argument, redundant-typehint-argument]
x: int | int | int | int

# No warning for type alias (yet !)
Q = int | int
QQ = Q | Q

q: Q | Q  # [redundant-typehint-argument]

# No warning for redundant types in compound type (yet !)
z: dict[int | int, str | str]

#  +1: [redundant-typehint-argument]
zz: dict[int | int, str | str] | dict[int | int, str | str]

# No warnings for redundant types in function signature (yet !)
def f(p: int | int) -> str | str: ...



# pylint: disable=missing-docstring,too-few-public-methods
"""Test typing.Final"""
import typing
from typing import Final

class Foo:
    """Class with class constants annotated with Final."""
    CLASS_CONST: Final[int] = 42
    CLASS_CONST2: Final = "const"
    variable: Final[str] = "invalid name"  # [invalid-name]
    CLASS_CONST3: typing.Final
    variable2: typing.Final[int]  # [invalid-name]
    CLASS_CONST4: Final[typing.ClassVar[str]] = "valid"

MODULE_CONST: Final = 1
module_var: typing.Final[str] = "const"  # [invalid-name]



"""Regression test for issue 5461
Crash on list comprehension with it used `type` as variable name

See: https://github.com/pylint-dev/pylint/issues/5461
"""
var = [type for type in [] if type["id"]]



# pylint: disable=missing-docstring
__revision__ = 0

class CustomBroadException(Exception):
    pass


class CustomNarrowException(CustomBroadException):
    pass


try:
    __revision__ += 1
except Exception: # [broad-exception-caught]
    print('error')


try:
    __revision__ += 1
except BaseException: # [broad-exception-caught]
    print('error')


try:
    __revision__ += 1
except ValueError:
    print('error')


try:
    __revision__ += 1
except CustomBroadException: # [broad-exception-caught]
    print('error')


try:
    __revision__ += 1
except CustomNarrowException:
    print('error')



"""Test pylint.extension.typing - consider-alternative-union-syntax

'py-version' needs to be set to >= '3.7' and 'runtime-typing=no'.
With 'from __future__ import annotations' present.
"""

# pylint: disable=missing-docstring,invalid-name,unused-argument,line-too-long
# pylint: disable=consider-using-alias,unnecessary-direct-lambda-call

# Disabled because of a bug with pypy 3.8 see
# https://github.com/pylint-dev/pylint/pull/7918#issuecomment-1352737369
# pylint: disable=multiple-statements

from __future__ import annotations
from dataclasses import dataclass
import typing
from typing import Dict, List, Optional, Union, TypedDict

var1: Union[int, str]  # [consider-alternative-union-syntax]
var2: List[Union[int, None]]  # [consider-alternative-union-syntax]
var3: Dict[str, typing.Union[int, str]]  # [consider-alternative-union-syntax]
var4: Optional[int]  # [consider-alternative-union-syntax]

Alias1 = Union[int, str]
Alias2 = List[Union[int, None]]
Alias3 = Dict[str, typing.Union[int, str]]
Alias4 = Optional[int]

def func1(
    arg1: Optional[int],  # [consider-alternative-union-syntax]
    **kwargs: Dict[str, Union[int, str]]  # [consider-alternative-union-syntax]
) -> Union[str, None]:  # [consider-alternative-union-syntax]
    pass

class Custom1(List[Union[str, int]]):
    pass

cast_variable = [1, 2, 3]
cast_variable = typing.cast(Union[List[int], None], cast_variable)

(lambda x: 2)(Optional[int])

class CustomNamedTuple(typing.NamedTuple):
    my_var: Union[int, str]  # [consider-alternative-union-syntax]

CustomTypedDict1 = TypedDict("CustomTypedDict1", my_var=Optional[int])

class CustomTypedDict2(TypedDict):
    my_var: Dict[str, List[Union[str, int]]]  # [consider-alternative-union-syntax]

@dataclass
class CustomDataClass:
    my_var: Optional[int]  # [consider-alternative-union-syntax]



"""Tests for missing-yield-doc and missing-yield-type-doc for Google style docstrings
with accept-no-yields-doc = no"""
# pylint: disable=missing-function-docstring, unused-argument, function-redefined
# pylint: disable=invalid-name, undefined-variable


def my_func(self):
    """This is a docstring.

    Yields:
        bool: Always False
    """
    yield False


def my_func(self):
    """This is a docstring.

    Yields:
        mymodule.Class: An object
    """
    yield mymodule.Class()


def my_func(self):
    """This is a docstring.

    Yields:
        list(:class:`mymodule.Class`): An object
    """
    yield [mymodule.Class()]


def my_func(self):  # [missing-yield-doc]
    """This is a docstring.

    Yields:
        list(:class:`mymodule.Class`):
    """
    yield [mymodule.Class()]


def my_func(self):  # [missing-yield-type-doc]
    """This is a docstring.

    Yields:
        Always False
    """
    yield False


def my_func(self):  # [missing-yield-doc]
    """This is a docstring.

    Yields:
        bool:
    """
    yield False


def my_func(self, doc_type):  # [missing-yield-doc, missing-yield-type-doc]
    """This is a docstring.

    Parameters:
        doc_type (str): Google
    """
    yield False



# pylint: disable=missing-docstring

from argparse import ArgumentParser # [unused-import]
from argparse import Namespace  # [unused-import]
import http  # [unused-import]
from http import HTTPStatus
import typing as t
from typing import Literal as Lit

# str inside Literal shouldn't be treated as names
example1: t.Literal["ArgumentParser", Lit["Namespace", "ArgumentParser"]]


def unused_variable_example():
    hello = "hello" # [unused-variable]
    world = "world" # [unused-variable]
    example2: Lit["hello", "world"] = "hello"
    return example2


# pylint shouldn't crash with the following strings in a type annotation context
example3: Lit["", " ", "?"] = "?"


# See https://peps.python.org/pep-0586/#literals-enums-and-forward-references
example4: t.Literal["http.HTTPStatus.OK", "http.HTTPStatus.NOT_FOUND"]
example5: "t.Literal[HTTPStatus.OK, HTTPStatus.NOT_FOUND]"



"""Regression test for https://github.com/pylint-dev/pylint/issues/6895"""
# pylint: disable=missing-class-docstring,too-few-public-methods
import argparse
class Cls:
    def meth(self):
        """Enable non-iterator-returned to produce the failure condition"""
        return argparse.Namespace(debug=True)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Unittest for the spelling checker."""

import astroid
import pytest

from pylint.checkers import spelling
from pylint.checkers.spelling import _get_enchant_dict_help
from pylint.testutils import CheckerTestCase, MessageTest, _tokenize_str, set_config

# try to create enchant dictionary
try:
    import enchant
except ImportError:
    enchant = None

spell_dict = None
if enchant is not None:
    try:
        enchant.Dict("en_US")
        spell_dict = "en_US"
    except enchant.DictNotFoundError:
        pass


class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-methods
    # This is a test case class, not sure why it would be relevant to have
    #   this pylint rule enforced for test case classes.
    CHECKER_CLASS = spelling.SpellingChecker

    skip_on_missing_package_or_dict = pytest.mark.skipif(
        spell_dict is None,
        reason="missing python-enchant package or missing spelling dictionaries",
    )

    def _get_msg_suggestions(self, word: str, count: int = 4) -> str:
        suggestions = "' or '".join(self.checker.spelling_dict.suggest(word)[:count])
        return f"'{suggestions}'"

    def test_spelling_dict_help_no_enchant(self) -> None:
        assert "both the python package and the system dep" in _get_enchant_dict_help(
            [], pyenchant_available=False
        )
        assert "need to install the system dep" in _get_enchant_dict_help(
            [], pyenchant_available=True
        )

    @skip_on_missing_package_or_dict
    def test_spelling_dict_help_enchant(self) -> None:
        assert "Available dictionaries: " in _get_enchant_dict_help(
            enchant.Broker().list_dicts(), pyenchant_available=True
        )

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_check_bad_coment(self) -> None:
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "coment",
                    "# bad coment",
                    "      ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.process_tokens(_tokenize_str("# bad coment"))

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    @set_config(max_spelling_suggestions=2)
    def test_check_bad_comment_custom_suggestion_count(self) -> None:
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "coment",
                    "# bad coment",
                    "      ^^^^^^",
                    self._get_msg_suggestions("coment", count=2),
                ),
            )
        ):
            self.checker.process_tokens(_tokenize_str("# bad coment"))

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_check_bad_docstring(self) -> None:
        stmt = astroid.extract_node('def fff():\n   """bad coment"""\n   pass')
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "bad coment",
                    "    ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_functiondef(stmt)

        stmt = astroid.extract_node('class Abc(object):\n   """bad coment"""\n   pass')
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "bad coment",
                    "    ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_shebangs(self) -> None:
        self.checker.process_tokens(_tokenize_str("#!/usr/bin/env python"))
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_python_coding_comments(self) -> None:
        self.checker.process_tokens(_tokenize_str("# -*- coding: utf-8 -*-"))
        assert not self.linter.release_messages()
        self.checker.process_tokens(_tokenize_str("# coding=utf-8"))
        assert not self.linter.release_messages()
        self.checker.process_tokens(_tokenize_str("# vim: set fileencoding=utf-8 :"))
        assert not self.linter.release_messages()
        # Now with a shebang first
        self.checker.process_tokens(
            _tokenize_str("#!/usr/bin/env python\n# -*- coding: utf-8 -*-")
        )
        assert not self.linter.release_messages()
        self.checker.process_tokens(
            _tokenize_str("#!/usr/bin/env python\n# coding=utf-8")
        )
        assert not self.linter.release_messages()
        self.checker.process_tokens(
            _tokenize_str("#!/usr/bin/env python\n# vim: set fileencoding=utf-8 :")
        )
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_top_level_pylint_enable_disable_comments(self) -> None:
        self.checker.process_tokens(
            _tokenize_str("# Line 1\n Line 2\n# pylint: disable=ungrouped-imports")
        )
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_words_with_numbers(self) -> None:
        self.checker.process_tokens(_tokenize_str("\n# 0ne\n# Thr33\n# Sh3ll"))
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_wiki_words(self) -> None:
        stmt = astroid.extract_node(
            'class ComentAbc(object):\n   """ComentAbc with a bad coment"""\n   pass'
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "ComentAbc with a bad coment",
                    "                     ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_camel_cased_words(self) -> None:
        stmt = astroid.extract_node(
            'class ComentAbc(object):\n   """comentAbc with a bad coment"""\n   pass'
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "comentAbc with a bad coment",
                    "                     ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

        # With just a single upper case letter in the end
        stmt = astroid.extract_node(
            'class ComentAbc(object):\n   """argumentN with a bad coment"""\n   pass'
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "argumentN with a bad coment",
                    "                     ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

        for ccn in (
            "xmlHttpRequest",
            "newCustomer",
            "newCustomerId",
            "innerStopwatch",
            "supportsIpv6OnIos",
            "affine3D",
        ):
            stmt = astroid.extract_node(
                f'class TestClass(object):\n   """{ccn} comment"""\n   pass'
            )
            self.checker.visit_classdef(stmt)
            assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_words_with_underscores(self) -> None:
        stmt = astroid.extract_node(
            'def fff(param_name):\n   """test param_name"""\n   pass'
        )
        self.checker.visit_functiondef(stmt)
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_email_address(self) -> None:
        self.checker.process_tokens(_tokenize_str("# uname@domain.tld"))
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_urls(self) -> None:
        self.checker.process_tokens(_tokenize_str("# https://github.com/rfk/pyenchant"))
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    @pytest.mark.parametrize(
        "type_comment",
        [
            "# type: (NotAWord) -> NotAWord",
            "# type: List[NotAWord] -> List[NotAWord]",
            "# type: Dict[NotAWord] -> Dict[NotAWord]",
            "# type: NotAWord",
            "# type: List[NotAWord]",
            "# type: Dict[NotAWord]",
            "# type: ImmutableList[Manager]",
            # will result in error: Invalid "type: ignore" comment  [syntax]
            # when analyzed with mypy 1.02
            "# type: ignore[attr-defined] NotAWord",
        ],
    )
    def test_skip_type_comments(self, type_comment: str) -> None:
        self.checker.process_tokens(_tokenize_str(type_comment))
        assert not self.linter.release_messages()

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_sphinx_directives(self) -> None:
        stmt = astroid.extract_node(
            'class ComentAbc(object):\n   """This is :class:`ComentAbc` with a bad coment"""\n   pass'
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "This is :class:`ComentAbc` with a bad coment",
                    "                                      ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_sphinx_directives_2(self) -> None:
        stmt = astroid.extract_node(
            'class ComentAbc(object):\n   """This is :py:attr:`ComentAbc` with a bad coment"""\n   pass'
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "coment",
                    "This is :py:attr:`ComentAbc` with a bad coment",
                    "                                        ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    @pytest.mark.parametrize(
        "prefix,suffix",
        (
            pytest.param("fmt", ": on", id="black directive to turn on formatting"),
            pytest.param("fmt", ": off", id="black directive to turn off formatting"),
            pytest.param("noqa", "", id="pycharm directive"),
            pytest.param("noqa", ":", id="flake8 / zimports directive"),
            pytest.param("nosec", "", id="bandit directive"),
            pytest.param("isort", ":skip", id="isort directive"),
            pytest.param("mypy", ":", id="mypy top of file directive"),
        ),
    )
    def test_tool_directives_handling(self, prefix: str, suffix: str) -> None:
        """We're not raising when the directive is at the beginning of comments,
        but we raise if a directive appears later in comment."""
        full_comment = f"# {prefix}{suffix} {prefix}"
        args = (
            prefix,
            full_comment,
            f"  {'^' * len(prefix)}",
            self._get_msg_suggestions(prefix),
        )
        with self.assertAddsMessages(
            MessageTest("wrong-spelling-in-comment", line=1, args=args)
        ):
            self.checker.process_tokens(_tokenize_str(full_comment))

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_code_flanked_in_double_backticks(self) -> None:
        full_comment = "# The function ``.qsize()`` .qsize()"
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "qsize",
                    full_comment,
                    "                 ^^^^^",
                    self._get_msg_suggestions("qsize"),
                ),
            )
        ):
            self.checker.process_tokens(_tokenize_str(full_comment))

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_skip_code_flanked_in_single_backticks(self) -> None:
        full_comment = "# The function `.qsize()` .qsize()"
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "qsize",
                    full_comment,
                    "                 ^^^^^",
                    self._get_msg_suggestions("qsize"),
                ),
            )
        ):
            self.checker.process_tokens(_tokenize_str(full_comment))

    @skip_on_missing_package_or_dict
    @set_config(
        spelling_dict=spell_dict,
        spelling_ignore_comment_directives="newdirective:,noqa",
    )
    def test_skip_directives_specified_in_pylintrc(self) -> None:
        full_comment = "# newdirective: do this newdirective"
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "newdirective",
                    full_comment,
                    "          ^^^^^^^^^^^^",
                    self._get_msg_suggestions("newdirective"),
                ),
            )
        ):
            self.checker.process_tokens(_tokenize_str(full_comment))

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_handle_words_joined_by_forward_slash(self) -> None:
        stmt = astroid.extract_node(
            '''
        class ComentAbc(object):
            """This is Comment/Abcz with a bad comment"""
            pass
        '''
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=3,
                args=(
                    "Abcz",
                    "This is Comment/Abcz with a bad comment",
                    "                ^^^^",
                    self._get_msg_suggestions("Abcz"),
                ),
            )
        ):
            self.checker.visit_classdef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_more_than_one_error_in_same_line_for_same_word_on_docstring(self) -> None:
        stmt = astroid.extract_node(
            'class ComentAbc(object):\n   """Check teh dummy comment teh"""\n   pass'
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "teh",
                    "Check teh dummy comment teh",
                    "      ^^^",
                    self._get_msg_suggestions("teh"),
                ),
            ),
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "teh",
                    "Check teh dummy comment teh",
                    "                        ^^^",
                    self._get_msg_suggestions("teh"),
                ),
            ),
        ):
            self.checker.visit_classdef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_more_than_one_error_in_same_line_for_same_word_on_comment(self) -> None:
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "coment",
                    "# bad coment coment",
                    "      ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            ),
            MessageTest(
                "wrong-spelling-in-comment",
                line=1,
                args=(
                    "coment",
                    "# bad coment coment",
                    "             ^^^^^^",
                    self._get_msg_suggestions("coment"),
                ),
            ),
        ):
            self.checker.process_tokens(_tokenize_str("# bad coment coment"))

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_docstring_lines_that_look_like_comments_1(self) -> None:
        stmt = astroid.extract_node(
            '''def f():
    """
    # msitake
    """'''
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=3,
                args=(
                    "msitake",
                    "    # msitake",
                    "      ^^^^^^^",
                    self._get_msg_suggestions("msitake"),
                ),
            )
        ):
            self.checker.visit_functiondef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_docstring_lines_that_look_like_comments_2(self) -> None:
        stmt = astroid.extract_node(
            '''def f():
    """# msitake"""'''
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=2,
                args=(
                    "msitake",
                    "# msitake",
                    "  ^^^^^^^",
                    self._get_msg_suggestions("msitake"),
                ),
            )
        ):
            self.checker.visit_functiondef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_docstring_lines_that_look_like_comments_3(self) -> None:
        stmt = astroid.extract_node(
            '''def f():
    """
# msitake
    """'''
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=3,
                args=(
                    "msitake",
                    "# msitake",
                    "  ^^^^^^^",
                    self._get_msg_suggestions("msitake"),
                ),
            )
        ):
            self.checker.visit_functiondef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_docstring_lines_that_look_like_comments_4(self) -> None:
        stmt = astroid.extract_node(
            '''def f():
    """
    # cat
    """'''
        )
        with self.assertAddsMessages():
            self.checker.visit_functiondef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_docstring_lines_that_look_like_comments_5(self) -> None:
        stmt = astroid.extract_node(
            '''def f():
    """
    msitake # cat
    """'''
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=3,
                args=(
                    "msitake",
                    "    msitake # cat",
                    "    ^^^^^^^",
                    self._get_msg_suggestions("msitake"),
                ),
            )
        ):
            self.checker.visit_functiondef(stmt)

    @skip_on_missing_package_or_dict
    @set_config(spelling_dict=spell_dict)
    def test_docstring_lines_that_look_like_comments_6(self) -> None:
        stmt = astroid.extract_node(
            '''def f():
    """
    cat # msitake
    """'''
        )
        with self.assertAddsMessages(
            MessageTest(
                "wrong-spelling-in-docstring",
                line=3,
                args=(
                    "msitake",
                    "    cat # msitake",
                    "          ^^^^^^^",
                    self._get_msg_suggestions("msitake"),
                ),
            )
        ):
            self.checker.visit_functiondef(stmt)



"""
Regression test for https://github.com/pylint-dev/pylint/issues/3979
"""

import os
from typing import TYPE_CHECKING, Any, Union

if TYPE_CHECKING:
    BasePathLike = os.PathLike[Any]  # <-- pylint used to emit E1136 here
else:
    BasePathLike = os.PathLike

Foo: Union[str, BasePathLike] = "bar"



"""Test detection of redundant nested calls to min/max functions"""

# pylint: disable=redefined-builtin,unnecessary-lambda-assignment

max(3, max({1: 2} | {i: i for i in range(4) if i}))  # [nested-min-max]
max(3, *{1: 2} | {i: i for i in range(4) if i})



"""Tests for missing-yield-doc and missing-yield-type-doc"""
# pylint: disable=missing-function-docstring, unused-argument, function-redefined

# Ignore no docstring
def my_func(self):
    yield False


# Ignore unrecognized style docstring
def my_func(self):
    """This is a docstring."""
    yield False



""" Test that import errors are detected. """
# pylint: disable=invalid-name, unused-import, bare-except, broad-except, wrong-import-order, wrong-import-position
import totally_missing # [import-error]

try:
    import maybe_missing
except ImportError:
    maybe_missing = None

try:
    import maybe_missing
except ModuleNotFoundError:
    maybe_missing = None

try:
    import maybe_missing_1
except (ImportError, SyntaxError):
    maybe_missing_1 = None

try:
    import maybe_missing_2 # [import-error]
except ValueError:
    maybe_missing_2 = None


try:
    if maybe_missing:
        import really_missing
except ImportError:
    pass


from functional.s.syntax.syntax_error import toto  # [no-name-in-module,syntax-error]


# Don't emit `import-error` or `no-name-in-module`
# if guarded behind `sys.version_info` or `typing.TYPE_CHECKING`
import sys
import typing
import typing as tp  # pylint: disable=reimported
from typing import TYPE_CHECKING


if sys.version_info >= (3, 9):
    import some_module
    from some_module import some_class
else:
    import some_module_alt

if sys.version_info[:2] >= (3, 9):
    import some_module
else:
    import some_module_alt


if typing.TYPE_CHECKING:
    import stub_import

if tp.TYPE_CHECKING:
    import stub_import

if TYPE_CHECKING:
    import stub_import
    from stub_import import stub_class


# Test ignore-modules option
from external_module import anything

from external_module.another_module import anything

import external_module

from fake_module.submodule import anything

from fake_module.submodule.deeper import anything

import foo, bar # [multiple-imports]

import foo
import bar

import mymodule_ignored
import mymodule.something_ignored
from mymodule.something_ignored import anything
import sys.something_ignored
from sys.something_ignored import anything

# Issues with contextlib.suppress reported in
# https://github.com/pylint-dev/pylint/issues/7270
import contextlib
with contextlib.suppress(ImportError):
    import foo2

with contextlib.suppress(ValueError):
    import foo2  # [import-error]

with contextlib.suppress(ImportError, ValueError):
    import foo2

with contextlib.suppress((ImportError, ValueError)):
    import foo2

with contextlib.suppress((ImportError,), (ValueError,)):
    import foo2

x = True
with contextlib.suppress(ImportError):
    if x:
        import foo2
    else:
        pass

with contextlib.suppress(ImportError):
    with contextlib.suppress(TypeError):
        import foo2



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt
# pylint: disable=missing-module-docstring, missing-function-docstring

from __future__ import annotations

import contextlib
import os
import pathlib
import shlex
import sys
from collections.abc import Sequence
from io import BufferedReader
from typing import Any, Protocol
from unittest.mock import MagicMock, mock_open, patch

import pytest

from pylint import run_pylint, run_pyreverse, run_symilar
from pylint.testutils import GenericTestReporter as Reporter
from pylint.testutils._run import _Run as Run
from pylint.testutils.utils import _test_cwd


class _RunCallable(Protocol):  # pylint: disable=too-few-public-methods
    def __call__(self, argv: Sequence[str] | None = None) -> None: ...


@pytest.mark.parametrize("runner", [run_pylint, run_pyreverse, run_symilar])
def test_runner(runner: _RunCallable, tmp_path: pathlib.Path) -> None:
    filepath = os.path.abspath(__file__)
    testargs = ["", filepath]
    with _test_cwd(tmp_path):
        with patch.object(sys, "argv", testargs):
            with pytest.raises(SystemExit) as err:
                runner()
            assert err.value.code == 0


@pytest.mark.parametrize("runner", [run_pylint, run_pyreverse, run_symilar])
def test_runner_with_arguments(runner: _RunCallable, tmp_path: pathlib.Path) -> None:
    """Check the runners with arguments as parameter instead of sys.argv."""
    filepath = os.path.abspath(__file__)
    testargs = [filepath]
    with _test_cwd(tmp_path):
        with pytest.raises(SystemExit) as err:
            runner(testargs)
        assert err.value.code == 0


def test_pylint_argument_deduplication(
    tmp_path: pathlib.Path, tests_directory: pathlib.Path
) -> None:
    """Check that the Pylint runner does not over-report on duplicate
    arguments.

    See https://github.com/pylint-dev/pylint/issues/6242 and
    https://github.com/pylint-dev/pylint/issues/4053
    """
    filepath = str(tests_directory / "functional/t/too/too_many_branches.py")
    testargs = shlex.split("--report n --score n --max-branches 13")
    testargs.extend([filepath] * 4)
    exit_stack = contextlib.ExitStack()
    exit_stack.enter_context(_test_cwd(tmp_path))
    exit_stack.enter_context(patch.object(sys, "argv", testargs))
    err = exit_stack.enter_context(pytest.raises(SystemExit))
    with exit_stack:
        run_pylint(testargs)
    assert err.value.code == 0


def test_pylint_run_jobs_equal_zero_dont_crash_with_cpu_fraction(
    tmp_path: pathlib.Path,
) -> None:
    """Check that the pylint runner does not crash if `pylint.lint.run._query_cpu`
    determines only a fraction of a CPU core to be available.
    """
    builtin_open = open

    def _mock_open(*args: Any, **kwargs: Any) -> BufferedReader:
        if args[0] == "/sys/fs/cgroup/cpu/cpu.cfs_quota_us":
            return mock_open(read_data=b"-1")(*args, **kwargs)  # type: ignore[no-any-return]
        if args[0] == "/sys/fs/cgroup/cpu/cpu.shares":
            return mock_open(read_data=b"2")(*args, **kwargs)  # type: ignore[no-any-return]
        return builtin_open(*args, **kwargs)  # type: ignore[no-any-return]

    pathlib_path = pathlib.Path

    def _mock_path(*args: str, **kwargs: Any) -> pathlib.Path:
        if args[0] == "/sys/fs/cgroup/cpu/cpu.shares":
            return MagicMock(is_file=lambda: True)
        return pathlib_path(*args, **kwargs)

    filepath = os.path.abspath(__file__)
    testargs = [filepath, "--jobs=0"]
    with _test_cwd(tmp_path):
        with pytest.raises(SystemExit) as err:
            with patch("builtins.open", _mock_open):
                with patch("pylint.lint.run.Path", _mock_path):
                    Run(testargs, reporter=Reporter())
        assert err.value.code == 0



# pylint: disable=no-else-raise,unsupported-membership-test,using-constant-test, comparison-of-constants

"""Checks use of "else if" triggers a refactor message"""
from typing import Union, Sequence, Any, Mapping


def my_function():
    """docstring"""
    myint = 2
    if myint > 5:
        pass
    else:
        if myint <= 5:  # [else-if-used]
            pass
        else:
            myint = 3
            if myint > 2:
                if myint > 3:
                    pass
                elif myint == 3:
                    pass
                elif myint < 3:
                    pass
                else:
                    if myint:  # [else-if-used]
                        pass
            else:
                if myint:
                    pass
                myint = 4


def _if_in_fstring_comprehension_with_elif(
    params: Union[Sequence[Any], Mapping[str, Any]]
):
    order = {}
    if "z" not in "false":
        raise TypeError(
            f" {', '.join(sorted(i for i in order or () if i not in params))}"
        )
    elif "z" not in "true":
        pass
    else:
        if "t" not in "false":  # [else-if-used]
            raise TypeError("d")
        else:
            if "y" in "life":  # [else-if-used]
                print("e")



# pylint: disable=missing-docstring, broad-exception-raised, too-few-public-methods, redefined-outer-name
# pylint: disable=consider-using-sys-exit, protected-access

import os
import signal
import sys

def func1():
    return 1
    print('unreachable') # [unreachable]

def func2():
    while 1:
        break
        print('unreachable') # [unreachable]

def func3():
    for i in (1, 2, 3):
        print(i)
        continue
        print('unreachable') # [unreachable]

def func4():
    raise Exception
    return 1 / 0 # [unreachable]


# https://github.com/pylint-dev/pylint/issues/4698
def func5():
    """Empty generator functions should be allowed."""
    return
    yield

def func6():
    """Add 'unreachable' if yield is followed by another node."""
    return
    yield
    print("unreachable")  # [unreachable]

def func7():
    sys.exit(1)
    var = 2 + 2  # [unreachable]
    print(var)

def func8():
    signal.signal(signal.SIGTERM, lambda *args: sys.exit(0))
    try:
        print(1)
    except KeyboardInterrupt:
        pass

class FalseExit:
    def exit(self, number):
        print(f"False positive this is not sys.exit({number})")

def func_false_exit():
    sys  = FalseExit()
    sys.exit(1)
    var = 2 + 2
    print(var)

def func9():
    os._exit()
    var = 2 + 2  # [unreachable]
    print(var)

def func10():
    exit()
    var = 2 + 2  # [unreachable]
    print(var)

def func11():
    quit()
    var = 2 + 2  # [unreachable]
    print(var)

incognito_function = sys.exit
def func12():
    incognito_function()
    var = 2 + 2  # [unreachable]
    print(var)



"""Regression test for issue 5408.

Recursion error for self-referencing class attribute.
See: https://github.com/pylint-dev/pylint/issues/5408
"""

# pylint: disable=missing-docstring, too-few-public-methods, invalid-name, inherit-non-class
# pylint: disable=no-self-argument


class MyInnerClass:
    ...


class MySubClass:
    inner_class = MyInnerClass


class MyClass:
    sub_class = MySubClass()


def get_unpatched_class(cls):
    return cls


def get_unpatched(item):
    lookup = get_unpatched_class if isinstance(item, type) else lambda item: None
    return lookup(item)


_Child = get_unpatched(MyClass.sub_class.inner_class)


class Child(_Child):
    def patch(cls):
        MyClass.sub_class.inner_class = cls



"""
Checks that magic values are not used in comparisons
"""
# pylint: disable=missing-docstring,invalid-name,too-few-public-methods,import-error,wrong-import-position

from enum import Enum


class Christmas(Enum):
    EVE = 25
    DAY = 26
    MONTH = 12


var = 7
if var > 5:  # [magic-value-comparison]
    pass

if (var + 5) > 10:  # [magic-value-comparison]
    pass

is_big = 100 < var  # [magic-value-comparison]

shouldnt_raise = 5 > 7   # [comparison-of-constants]
shouldnt_raise = var == '__main__'
shouldnt_raise = var == 1
shouldnt_raise = var == 0
shouldnt_raise = var == -1
shouldnt_raise = var == True  # [singleton-comparison]
shouldnt_raise = var == False  # [singleton-comparison]
shouldnt_raise = var == None  # [singleton-comparison]
celebration_started = Christmas.EVE.value == Christmas.MONTH.value
shouldnt_raise = var == ""

shouldnt_raise = var == '\n'
shouldnt_raise = var == '\\b'

should_escape = var == "foo\nbar"  # [magic-value-comparison]



"""https://github.com/pylint-dev/pylint/issues/85"""
def main():
    """When evaluating finally blocks, assume try statements fail."""
    try:
        res = 1 / 0
        res = 42
    finally:
        print(res)  # [used-before-assignment]
    print(res)


def try_except_finally():
    """When evaluating finally blocks, assume try statements fail."""
    try:
        res = 1 / 0
        res = 42
    except ZeroDivisionError:
        print()
    finally:
        print(res)  # [used-before-assignment]
    print(res)


def try_except_finally_assignment_in_final_block():
    """Assignment of the name in the final block does not warn."""
    try:
        res = 1 / 0
        res = 42
    except ZeroDivisionError:
        print()
    finally:
        res = 999
        print(res)
    print(res)


def try_except_finally_assignment_in_both_try_and_except():
    """Assignment of the name in both try and except blocks is fine."""
    try:
        res = 1 / 0
    except ZeroDivisionError:
        res = 0
    finally:
        print(res)


def try_except_finally_nested_try_finally_in_try():
    """Don't confuse assignments in different finally statements where
    one is nested inside a try.
    """
    try:
        try:
            res = 1 / 0
        finally:
            print(res)  # [used-before-assignment]
        print(1 / 0)
    except ZeroDivisionError:
        print()
    finally:
        res = 999  # this assignment could be confused for that above
        print(res)
    print(res)


def try_except_finally_nested_in_finally():
    """Until Pylint comes to a consensus on requiring all except handlers to
    define a name, raise, or return (https://github.com/pylint-dev/pylint/issues/5524),
    Pylint assumes statements in try blocks succeed when accessed *after*
    except or finally blocks and fail when accessed *in* except or finally
    blocks.)
    """
    try:
        outer_times = 1
    finally:
        try:
            inner_times = 1
        except TypeError:
            pass
        finally:
            print(outer_times)  # [used-before-assignment]
        print(inner_times)  # see docstring: might emit in a future version


def try_except_finally_nested_in_finally_2():
    """Neither name is accessed after a finally block."""
    try:
        outer_times = 1
    finally:
        try:
            inner_times = 1
        except TypeError:
            pass
        finally:
            print(inner_times)  # [used-before-assignment]
        print(outer_times)  # [used-before-assignment]


def try_except_finally_nested_in_finally_3():
    """One name is never accessed after a finally block, but just emit
    once per name.
    """
    try:
        outer_times = 1
    finally:
        try:
            inner_times = 1
        except TypeError:
            pass
        finally:
            print(inner_times)  # [used-before-assignment]
            print(outer_times)  # [used-before-assignment]
        print(inner_times)
        # used-before-assignment is only raised once per name
        print(outer_times)


def try_except_finally_nested_in_finally_4():
    """Triple nesting: don't assume direct parentages of outer try/finally
    and inner try/finally.
    """
    try:
        outer_times = 1
    finally:
        try:
            pass
        finally:
            try:
                inner_times = 1
            except TypeError:
                pass
            finally:
                print(inner_times)  # [used-before-assignment]
                print(outer_times)  # [used-before-assignment]
            print(inner_times)
            # used-before-assignment is only raised once per name
            print(outer_times)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import contextlib
import importlib
import os
import shutil
import sys
import tempfile
from collections.abc import Iterator
from pathlib import Path
from unittest import mock

import pytest
from pytest import CaptureFixture

from pylint import config, testutils
from pylint.config.find_default_config_files import _cfg_has_config, _toml_has_config
from pylint.lint.run import Run


@pytest.fixture
def pop_pylintrc() -> None:
    """Remove the PYLINTRC environment variable"""
    os.environ.pop("PYLINTRC", None)


# pylint: disable=duplicate-code
if os.name == "java":
    if os.name == "nt":
        HOME = "USERPROFILE"
    else:
        HOME = "HOME"
elif sys.platform == "win32":
    HOME = "USERPROFILE"
else:
    HOME = "HOME"


@contextlib.contextmanager
def fake_home() -> Iterator[None]:
    """Fake a home directory."""
    folder = tempfile.mkdtemp("fake-home")
    old_home = os.environ.get(HOME)
    try:
        os.environ[HOME] = folder
        yield
    finally:
        os.environ.pop("PYLINTRC", "")
        if old_home is None:
            del os.environ[HOME]
        else:
            os.environ[HOME] = old_home
        shutil.rmtree(folder, ignore_errors=True)


# pylint: enable=duplicate-code


@contextlib.contextmanager
def tempdir() -> Iterator[str]:
    """Create a temp directory and change the current location to it.

    This is supposed to be used with a *with* statement.
    """
    tmp = tempfile.mkdtemp()

    # Get real path of tempfile, otherwise test fail on mac os x
    current_dir = os.getcwd()
    os.chdir(tmp)
    abs_tmp = os.path.abspath(".")

    try:
        yield abs_tmp
    finally:
        os.chdir(current_dir)
        shutil.rmtree(abs_tmp)


@pytest.mark.usefixtures("pop_pylintrc")
def test_pylintrc() -> None:
    """Test that the environment variable is checked for existence."""
    with fake_home():
        current_dir = os.getcwd()
        os.chdir(os.path.dirname(os.path.abspath(sys.executable)))
        try:
            assert not list(config.find_default_config_files())
            os.environ["PYLINTRC"] = os.path.join(tempfile.gettempdir(), ".pylintrc")
            assert not list(config.find_default_config_files())
            os.environ["PYLINTRC"] = "."
            assert not list(config.find_default_config_files())
        finally:
            os.chdir(current_dir)
            importlib.reload(config)


@pytest.mark.usefixtures("pop_pylintrc")
def test_pylintrc_parentdir() -> None:
    """Test that the first pylintrc we find is the first parent directory."""
    # pylint: disable=duplicate-code
    with tempdir() as chroot:
        chroot_path = Path(chroot)
        testutils.create_files(
            [
                "a/pylintrc",
                "a/b/__init__.py",
                "a/b/pylintrc",
                "a/b/c/__init__.py",
                "a/b/c/d/__init__.py",
                "a/b/c/d/e/.pylintrc",
            ]
        )

        with fake_home():
            assert not list(config.find_default_config_files())

        results = {
            "a": chroot_path / "a" / "pylintrc",
            "a/b": chroot_path / "a" / "b" / "pylintrc",
            "a/b/c": chroot_path / "a" / "b" / "pylintrc",
            "a/b/c/d": chroot_path / "a" / "b" / "pylintrc",
            "a/b/c/d/e": chroot_path / "a" / "b" / "c" / "d" / "e" / ".pylintrc",
        }
        for basedir, expected in results.items():
            os.chdir(chroot_path / basedir)
            assert next(config.find_default_config_files()) == expected


@pytest.mark.usefixtures("pop_pylintrc")
def test_pylintrc_toml_parentdir() -> None:
    """Test that the first pylintrc.toml we find is the first parent directory."""
    # pylint: disable=duplicate-code
    with tempdir() as chroot:
        chroot_path = Path(chroot)
        files = [
            "a/pylintrc.toml",
            "a/b/__init__.py",
            "a/b/pylintrc.toml",
            "a/b/c/__init__.py",
            "a/b/c/d/__init__.py",
            "a/b/c/d/e/.pylintrc.toml",
        ]
        testutils.create_files(files)
        for config_file in files:
            if config_file.endswith("pylintrc.toml"):
                with open(config_file, "w", encoding="utf-8") as fd:
                    fd.write('[tool.pylint."messages control"]\n')

        with fake_home():
            assert not list(config.find_default_config_files())

        results = {
            "a": chroot_path / "a" / "pylintrc.toml",
            "a/b": chroot_path / "a" / "b" / "pylintrc.toml",
            "a/b/c": chroot_path / "a" / "b" / "pylintrc.toml",
            "a/b/c/d": chroot_path / "a" / "b" / "pylintrc.toml",
            "a/b/c/d/e": chroot_path / "a" / "b" / "c" / "d" / "e" / ".pylintrc.toml",
        }
        for basedir, expected in results.items():
            os.chdir(chroot_path / basedir)
            assert next(config.find_default_config_files()) == expected


@pytest.mark.usefixtures("pop_pylintrc")
def test_pyproject_toml_parentdir() -> None:
    """Test the search of pyproject.toml file in parent directories"""
    with tempdir() as chroot:
        with fake_home():
            chroot_path = Path(chroot)
            files = [
                "pyproject.toml",
                "git/pyproject.toml",
                "git/a/pyproject.toml",
                "git/a/.git",
                "git/a/b/c/__init__.py",
                "hg/pyproject.toml",
                "hg/a/pyproject.toml",
                "hg/a/.hg",
                "hg/a/b/c/__init__.py",
                "none/sub/__init__.py",
            ]
            testutils.create_files(files)
            for config_file in files:
                if config_file.endswith("pyproject.toml"):
                    with open(config_file, "w", encoding="utf-8") as fd:
                        fd.write('[tool.pylint."messages control"]\n')
            results = {
                "": chroot_path / "pyproject.toml",
                "git": chroot_path / "git" / "pyproject.toml",
                "git/a": chroot_path / "git" / "a" / "pyproject.toml",
                "git/a/b": chroot_path / "git" / "a" / "pyproject.toml",
                "git/a/b/c": chroot_path / "git" / "a" / "pyproject.toml",
                "hg": chroot_path / "hg" / "pyproject.toml",
                "hg/a": chroot_path / "hg" / "a" / "pyproject.toml",
                "hg/a/b": chroot_path / "hg" / "a" / "pyproject.toml",
                "hg/a/b/c": chroot_path / "hg" / "a" / "pyproject.toml",
                "none": chroot_path / "pyproject.toml",
                "none/sub": chroot_path / "pyproject.toml",
            }
            for basedir, expected in results.items():
                os.chdir(chroot_path / basedir)
                assert next(config.find_default_config_files(), None) == expected


@pytest.mark.usefixtures("pop_pylintrc")
def test_pylintrc_parentdir_no_package() -> None:
    """Test that we don't find a pylintrc in sub-packages."""
    with tempdir() as chroot:
        with fake_home():
            chroot_path = Path(chroot)
            testutils.create_files(
                ["a/pylintrc", "a/b/pylintrc", "a/b/c/d/__init__.py"]
            )
            results = {
                "a": chroot_path / "a" / "pylintrc",
                "a/b": chroot_path / "a" / "b" / "pylintrc",
                "a/b/c": None,
                "a/b/c/d": None,
            }
            for basedir, expected in results.items():
                os.chdir(chroot_path / basedir)
                assert next(config.find_default_config_files(), None) == expected


@pytest.mark.usefixtures("pop_pylintrc")
def test_verbose_output_no_config(capsys: CaptureFixture) -> None:
    """Test that we print a log message in verbose mode with no file."""
    with tempdir() as chroot:
        with fake_home():
            chroot_path = Path(chroot)
            testutils.create_files(["a/b/c/d/__init__.py"])
            os.chdir(chroot_path / "a/b/c")
            with pytest.raises(SystemExit):
                Run(["--verbose"])
            out = capsys.readouterr()
            assert "No config file found, using default configuration" in out.err


@pytest.mark.usefixtures("pop_pylintrc")
def test_verbose_abbreviation(capsys: CaptureFixture) -> None:
    """Test that we correctly handle an abbreviated pre-processable option."""
    with tempdir() as chroot:
        with fake_home():
            chroot_path = Path(chroot)
            testutils.create_files(["a/b/c/d/__init__.py"])
            os.chdir(chroot_path / "a/b/c")
            with pytest.raises(SystemExit):
                Run(["--ve"])
            out = capsys.readouterr()
            # This output only exists when launched in verbose mode
            assert "No config file found, using default configuration" in out.err


@pytest.mark.parametrize(
    "content,expected",
    [
        ["", False],
        ["(not toml valid)", False],
        [
            """
[build-system]
requires = ["setuptools ~= 58.0", "cython ~= 0.29.0"]
""",
            False,
        ],
        [
            """
[tool.pylint]
missing-member-hint = true
""",
            True,
        ],
    ],
)
def test_toml_has_config(content: str, expected: bool, tmp_path: Path) -> None:
    """Test that a toml file has a pylint config."""
    fake_toml = tmp_path / "fake.toml"
    with open(fake_toml, "w", encoding="utf8") as f:
        f.write(content)
    assert _toml_has_config(fake_toml) == expected


@pytest.mark.parametrize(
    "content,expected",
    [
        ["", False],
        ["(not valid .cfg)", False],
        [
            """
[metadata]
name = pylint
""",
            False,
        ],
        [
            """
[metadata]
name = pylint

[pylint.messages control]
disable = logging-not-lazy,logging-format-interpolation
""",
            True,
        ],
    ],
)
def test_cfg_has_config(content: str, expected: bool, tmp_path: Path) -> None:
    """Test that a cfg file has a pylint config."""
    fake_cfg = tmp_path / "fake.cfg"
    with open(fake_cfg, "w", encoding="utf8") as f:
        f.write(content)
    assert _cfg_has_config(fake_cfg) == expected


def test_non_existent_home() -> None:
    """Test that we handle a non-existent home directory.

    Reported in https://github.com/pylint-dev/pylint/issues/6802.
    """
    with mock.patch("pathlib.Path.home", side_effect=RuntimeError):
        current_dir = os.getcwd()
        os.chdir(os.path.dirname(os.path.abspath(sys.executable)))

        assert not list(config.find_default_config_files())

        os.chdir(current_dir)


def test_permission_error() -> None:
    """Test that we handle PermissionError correctly in find_default_config_files.

    Reported in https://github.com/pylint-dev/pylint/issues/7169.
    """
    with mock.patch("pathlib.Path.is_file", side_effect=PermissionError):
        list(config.find_default_config_files())



"""Test related function for generic alias.

Any solution should not change the behavior of
- `__getitem__`
- `__class_getitem__`
- `metaclass=ABCMeta`
"""
# flake8: noqa
# pylint: disable=missing-docstring,pointless-statement,expression-not-assigned
# pylint: disable=too-few-public-methods,multiple-statements,line-too-long
from abc import ABCMeta, abstractmethod
import typing

GenericAlias = type(list[int])


class ClsUnsubscriptable:
    def __init__(self):
        pass

class ClsGetItem:
    def __init__(self):
        self.var = [1, 2, 3, 4]
    def __getitem__(self, item):
        return self.var[item]

class ClsClassGetItem:
    def __init__(self):
        pass
    __class_getitem__ = classmethod(GenericAlias)

class ClsList(typing.List):
    pass


ClsUnsubscriptable()[1]  # [unsubscriptable-object]
ClsUnsubscriptable[int]  # [unsubscriptable-object]

ClsGetItem()[1]
ClsGetItem[int]  # [unsubscriptable-object]

ClsClassGetItem()[1]  # [unsubscriptable-object]
ClsClassGetItem[int]

# subscriptable because of inheritance
ClsList([0, 1, 2])[1]
ClsList[int]


class ClsAbstract(metaclass=ABCMeta):
    @abstractmethod
    def abstract_method(self):
        pass

class Derived(ClsAbstract):  # [abstract-method]
    pass



# Regression test for https://github.com/pylint-dev/astroid/pull/1734
# The following should lint just fine
# Fixed in https://github.com/pylint-dev/astroid/pull/1743

# pylint: disable=missing-docstring,invalid-name

from enum import Enum

class Test(Enum):
    LOADED = "loaded", True
    SETUP_ERROR = "setup_error", True

    _recoverable: bool

    def __new__(cls, value: str, recoverable: bool):
        obj = object.__new__(cls)
        obj._value_ = value
        obj._recoverable = recoverable
        return obj

    @property
    def recoverable(self) -> bool:
        """Get if the state is recoverable."""
        return self._recoverable



"""Tests for super-init-not-called."""
# pylint: disable=too-few-public-methods

from typing_extensions import Protocol as ExtensionProtocol


class TestProto(ExtensionProtocol):
    """A protocol without __init__ using Protocol from typing_extensions."""


class TestParent(TestProto):
    """An implementation."""

    def __init__(self):
        ...


class TestChild(TestParent):
    """An implementation which should call the init of TestParent."""

    def __init__(self):  # [super-init-not-called]
        ...



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from pathlib import Path

import pytest
from _pytest.capture import CaptureFixture

from pylint import run_pylint


def test_run_pylint_with_invalid_argument(capsys: CaptureFixture[str]) -> None:
    """Check that appropriate exit code is used with invalid argument."""
    with pytest.raises(SystemExit) as ex:
        run_pylint(["--never-use-this"])
    captured = capsys.readouterr()
    assert captured.err.startswith("usage: pylint [options]")
    assert ex.value.code == 32


def test_run_pylint_with_invalid_argument_in_config(
    capsys: CaptureFixture[str], tmp_path: Path
) -> None:
    """Check that appropriate exit code is used with an ambiguous
    argument in a config file.
    """
    test_file = tmp_path / "testpylintrc"
    with open(test_file, "w", encoding="utf-8") as f:
        f.write("[MASTER]\nno=")

    with pytest.raises(SystemExit) as ex:
        run_pylint(["--rcfile", f"{test_file}"])
    captured = capsys.readouterr()
    assert captured.err.startswith("usage: pylint [options]")
    assert ex.value.code == 32



"""Tests for method-cache-max-size-none"""
# pylint: disable=missing-function-docstring, reimported, too-few-public-methods
# pylint: disable=missing-class-docstring, function-redefined

import functools
import functools as aliased_functools
from functools import cache
from functools import cache as aliased_cache


@cache
def my_func(param):
    return param + 1


class MyClassWithMethods:
    @cache
    @staticmethod
    def my_func(param):
        return param + 1

    @cache
    @classmethod
    def my_func(cls, param):
        return param + 1

    @cache  # [method-cache-max-size-none]
    def my_func(self, param):
        return param + 1

    @functools.cache  # [method-cache-max-size-none]
    def my_func(self, param):
        return param + 1

    @aliased_functools.cache  # [method-cache-max-size-none]
    def my_func(self, param):
        return param + 1

    @aliased_cache  # [method-cache-max-size-none]
    def my_func(self, param):
        return param + 1

    # Check double decorating to check robustness of checker itself
    @functools.lru_cache(maxsize=1)
    @aliased_cache  # [method-cache-max-size-none]
    def my_func(self, param):
        return param + 1



# pylint: disable=unused-argument, missing-docstring, line-too-long, too-few-public-methods
import enum


class Condiment(enum.Enum):
    CINAMON = 1
    SUGAR = 2

class Fruit:
    def brew(self, fruit_name: str):
        print(f"Brewing a fruit named {fruit_name}")

    def eat_with_condiment(self, fruit_name:str, condiment: Condiment):
        print(f"Eating a fruit named {fruit_name} with {condiment}")

class Orange(Fruit):
    def brew(self, orange_name: str): # [arguments-renamed]
        print(f"Brewing an orange named {orange_name}")

    def eat_with_condiment(self, orange_name: str, condiment: Condiment): #[arguments-renamed]
        print(f"Eating a fruit named {orange_name} with {condiment}")

class Banana(Fruit):
    def brew(self, fruit_name: bool): # No warning here
        print(f"Brewing a banana named {fruit_name}")

    def eat_with_condiment(self, fruit_name: str, condiment: Condiment, error: str): # [arguments-differ]
        print(f"Eating a fruit named {fruit_name} with {condiment}")

class Parent:

    def test(self, arg):
        return arg + 1

    def kwargs_test(self, arg, *, var1, var2):
        print(f"keyword parameters are {var1} and {var2}.")

class Child(Parent):

    def test(self, arg1): # [arguments-renamed]
        return arg1 + 1

    def kwargs_test(self, arg, *, value1, var2): #[arguments-differ]
        print(f"keyword parameters are {value1} and {var2}.")

class Child2(Parent):

    def test(self, var): # [arguments-renamed]
        return var + 1

    def kwargs_test(self, *, var1, kw2): #[arguments-differ]
        print(f"keyword parameters are {var1} and {kw2}.")

class ParentDefaults:

    def test1(self, arg, barg):
        print(f"Argument values are {arg} and {barg}")

    def test2(self, arg, barg):
        print(f"Argument values are {arg} and {barg}!")

    def test3(self, arg1, arg2):
        print(f"arguments: {arg1} {arg2}")

class ChildDefaults(ParentDefaults):

    def test1(self, arg, param2): # [arguments-renamed]
        print(f"Argument values are {arg} and {param2}")

    def test2(self, _arg, ignored_barg): # no error here
        print(f"Argument value is {_arg}")

    def test3(self, dummy_param, arg2): # no error here
        print(f"arguments: {arg2}")

# Check for crash on method definitions not at top level of class
# https://github.com/pylint-dev/pylint/issues/5648
class FruitConditional:

    define_eat = True

    def brew(self, fruit_name: str):
        print(f"Brewing a fruit named {fruit_name}")

    if define_eat:
        def eat_with_condiment(self, fruit_name:str, condiment: Condiment):
            print(f"Eating a fruit named {fruit_name} with {condiment}")

class FruitOverrideConditional(FruitConditional):

    fruit = "orange"
    override_condiment = True

    if fruit == "orange":
        def brew(self, orange_name: str): # [arguments-renamed]
            print(f"Brewing an orange named {orange_name}")

        if override_condiment:
            def eat_with_condiment(self, fruit_name: str, condiment: Condiment, error: str): # [arguments-differ]
                print(f"Eating a fruit named {fruit_name} with {condiment}")



"""Tests for dataclass attributes with basic type annotations.

Tests for regressions from https://github.com/pylint-dev/astroid/pull/1126
"""

# pylint: disable=missing-docstring,too-few-public-methods,pointless-statement,redefined-builtin, fixme

# Disabled because of a bug with pypy 3.8 see
# https://github.com/pylint-dev/pylint/pull/7918#issuecomment-1352737369
# pylint: disable=multiple-statements

from dataclasses import dataclass
from typing import Callable, Dict, List, Optional


class Dummy:
    pass


@dataclass
class MyClass:
    # Attribute inference does not support Optional, so Uninferable is yielded
    # This should not trigger any type errors from pylint
    attr0: Optional[Dummy]

    attr1: int
    attr2: str
    attr3: Callable[[int], int]
    attr4: List[int]
    attr5: Dict[str, str]


obj = MyClass(None, 1, 'hi', lambda x: x, [], {})

lst = [0, 1, 2]
print(lst[obj.attr0])
print(lst[obj.attr1])
print(lst[obj.attr2])  # [invalid-sequence-index]

print(lst[obj.attr0::])
print(lst[obj.attr1::])
print(lst[obj.attr2::])  # [invalid-slice-index]

obj.attr0(100)
obj.attr1(100)  # [not-callable]
obj.attr3(100)

print(-obj.attr0)
print(-obj.attr1)
print(-obj.attr2)  # [invalid-unary-operand-type]

print(1 + obj.attr0)
print(1 + obj.attr1)
print(1 + obj.attr2)  # Should be an error here once unsupported-binary-operation is enabled

print(1 in obj.attr0)
print(1 in obj.attr1)  # [unsupported-membership-test]
print(1 in obj.attr4)
print('hi' in obj.attr5)

print(obj.attr0[1])
print(obj.attr1[1])  # [unsubscriptable-object]
print(obj.attr4[1])
print(obj.attr5['hi'])

obj.attr0[1] = 1
obj.attr1[1] = 1  # [unsupported-assignment-operation]
obj.attr4[1] = 1
obj.attr5['hi'] = 'bye'

del obj.attr0[1]
del obj.attr1[1]  # [unsupported-delete-operation]
del obj.attr4[1]
del obj.attr5['hi']


class Manager:
    def __enter__(self):
        pass

    def __exit__(self, type_, value, traceback):
        pass


@dataclass
class MyClass2:
    attr0: Optional[Dummy]
    attr1: Manager
    attr2: str


obj2 = MyClass2(None, Manager(), 'hi')
with obj2.attr0:
    pass
with obj2.attr1:
    pass
with obj2.attr2:  # [not-context-manager]
    pass


class Test1(metaclass=obj.attr0):
    pass


class Test2(metaclass=obj.attr1):  # [invalid-metaclass]
    pass


{}[obj.attr0] = 1
{}[obj.attr1] = 1
{}[obj.attr5] = 1  # [unhashable-member]

for k, v in obj.attr5:  # TODO: Should be a dict-iter-missing-items error
    print(k, v)

__name__ = obj.attr0
__name__ = obj.attr1  # TODO: Should be a non-str-assignment-to-dunder-name error
__name__ = obj.attr2

print(isinstance(1, obj.attr0))
print(isinstance(1, obj.attr1))  # [isinstance-second-argument-not-valid-type]



"""Typing constants are actually implemented as functions, but they
raise when called, so Pylint uses that to avoid false positives for
comparison-with-callable.
"""
from typing import Any, Optional


def check_any(type_) -> bool:
    """See https://github.com/pylint-dev/pylint/issues/5557"""
    return type_ == Any


def check_optional(type_) -> bool:
    """
    Unlike Any, Optional does not raise in its body.
    It raises via its decorator: typing._SpecialForm.__call__()
    """
    return type_ == Optional



"""Tests for missing-return-doc and missing-return-type-doc for Numpy style docstrings"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, too-few-public-methods, disallowed-name
import abc


def my_func(self):
    """find_numpy_returns

    Returns
    -------
    bool
        Always False
    """
    return False


def my_func(self):
    """find_numpy_returns_with_of

    Returns
    -------
    :obj:`list` of :obj:`str`
        List of strings
    """
    return ["hi", "bye"]


def my_func(self, doc_type):
    """ignores_numpy_return_none

    Arguments
    ---------
    doc_type : str
        Numpy
    """
    return


def my_func(self):
    """finds_numpy_return_custom_class

    Returns
    -------
        mymodule.Class
            An object
    """
    return mymodule.Class()


def my_func(self):
    """finds_numpy_return_list_of_custom_class

    Returns
    -------
        list(:class:`mymodule.Class`)
            An object
    """
    return [mymodule.Class()]


def my_func(self):  # [redundant-returns-doc]
    """warns_numpy_redundant_return_doc

    Returns
    -------
        int
            One
    """
    return None


def my_func(self):  # [redundant-returns-doc]
    """warns_numpy_redundant_rtype_doc

    Returns
    -------
        int
    """
    return None


def my_func(self):
    """ignores_numpy_redundant_return_doc_multiple_returns

    Returns
    -------
        int
            One
        None
            Sometimes
    """
    if a_func():
        return None
    return 1


def my_func(self):  # [redundant-returns-doc]
    """warns_numpy_redundant_return_doc_yield

    Returns
    -------
        int
            One
    """
    yield 1


class Foo:
    """test_ignores_return_in_abstract_method_numpy
    Example of an abstract method documenting the return type that an
    implementation should return."""

    @abc.abstractmethod
    def foo(self):
        """docstring ...

        Returns
        -------
        int
            Ten
        """
        return 10


class Foo:
    """test_ignores_return_in_abstract_method_numpy_2
    Example of a method documenting the return type that an
    implementation should return."""

    def foo(self, arg):
        """docstring ...

        Parameters
        ----------
        arg : int
            An argument.
        """
        raise NotImplementedError()


class Foo:
    """test_ignores_ignored_argument_names_numpy
    Example of a method documenting the return type that an
    implementation should return.
    """

    def foo(self, arg, _):
        """docstring ...

        Parameters
        ----------
        arg : int
            An argument.
        """


class Foo:
    """test_useless_docs_ignored_argument_names_numpy
    Example of a method documenting the return type that an
    implementation should return.
    """

    def foo(self, arg, _, _ignored):  # [useless-type-doc, useless-param-doc]
        """docstring ...

        Parameters
        ----------
        arg : int
            An argument.

        _ : float
            Another argument.

        _ignored :
            Ignored Argument
        """



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Test the primer commands. """
from __future__ import annotations

import sys
from pathlib import Path
from unittest.mock import patch

import pytest
from _pytest.capture import CaptureFixture

from pylint.constants import IS_PYPY
from pylint.testutils._primer.primer import Primer

HERE = Path(__file__).parent
TEST_DIR_ROOT = HERE.parent.parent
PRIMER_DIRECTORY = TEST_DIR_ROOT / ".pylint_primer_tests/"
PACKAGES_TO_PRIME_PATH = TEST_DIR_ROOT / "primer/packages_to_prime.json"
FIXTURES_PATH = HERE / "fixtures"

PRIMER_CURRENT_INTERPRETER = (3, 11)

DEFAULT_ARGS = ["python tests/primer/__main__.py", "compare", "--commit=v2.14.2"]


@pytest.mark.parametrize("args", [[], ["wrong_command"]])
def test_primer_launch_bad_args(args: list[str], capsys: CaptureFixture) -> None:
    with pytest.raises(SystemExit):
        with patch("sys.argv", ["python tests/primer/__main__.py", *args]):
            Primer(PRIMER_DIRECTORY, PACKAGES_TO_PRIME_PATH).run()
    out, err = capsys.readouterr()
    assert not out
    assert "usage: Pylint Primer" in err


@pytest.mark.skipif(
    sys.version_info[:2] != PRIMER_CURRENT_INTERPRETER or IS_PYPY,
    reason=(
        "Primers are internal and will always be run for only one interpreter (currently"
        f" {PRIMER_CURRENT_INTERPRETER})"
    ),
)
class TestPrimer:
    @pytest.mark.parametrize(
        "directory",
        [
            pytest.param(p, id=str(p.relative_to(FIXTURES_PATH)))
            for p in FIXTURES_PATH.iterdir()
            if p.is_dir() and p.name != "batched"  # tested separately
        ],
    )
    def test_compare(self, directory: Path) -> None:
        """Test for the standard case.

        Directory in 'fixtures/' with 'main.json', 'pr.json' and 'expected.txt'."""
        self.__assert_expected(directory)

    def test_compare_batched(self) -> None:
        fixture = FIXTURES_PATH / "batched"
        self.__assert_expected(
            fixture,
            fixture / "main_BATCHIDX.json",
            fixture / "pr_BATCHIDX.json",
            batches=2,
        )

    def test_truncated_compare(self) -> None:
        """Test for the truncation of comments that are too long."""
        max_comment_length = 525
        directory = FIXTURES_PATH / "message_changed"
        with patch(
            "pylint.testutils._primer.primer_compare_command.MAX_GITHUB_COMMENT_LENGTH",
            max_comment_length,
        ):
            content = self.__assert_expected(
                directory, expected_file=directory / "expected_truncated.txt"
            )
        assert len(content) < max_comment_length

    @staticmethod
    def __assert_expected(
        directory: Path,
        main: Path | None = None,
        pr: Path | None = None,
        expected_file: Path | None = None,
        batches: int = 0,
    ) -> str:
        if main is None:
            main = directory / "main.json"
        if pr is None:
            pr = directory / "pr.json"
        if expected_file is None:
            expected_file = directory / "expected.txt"
        new_argv = [*DEFAULT_ARGS, f"--base-file={main}", f"--new-file={pr}"]
        if batches:
            new_argv.append(f"--batches={batches}")
        with patch("sys.argv", new_argv):
            Primer(PRIMER_DIRECTORY, PACKAGES_TO_PRIME_PATH).run()
        with open(PRIMER_DIRECTORY / "comment.txt", encoding="utf8") as f:
            content = f.read()
        with open(expected_file, encoding="utf8") as f:
            expected = f.read()
        # rstrip so the expected.txt can end with a newline
        assert content == expected.rstrip("\n")
        return content



# pylint: disable=missing-docstring,invalid-name,too-few-public-methods

# https://github.com/pylint-dev/pylint/issues/2822
# Base should be subscriptable, even with ABCMeta as metaclass
from abc import ABC, ABCMeta
from typing import Generic, TypeVar

T = TypeVar("T")

class Base(Generic[T], metaclass=ABCMeta):
    """Base"""

class Impl(Base[str]):
    """Impl"""


# https://github.com/pylint-dev/astroid/issues/942
Anything = TypeVar("Anything")
MoreSpecific = TypeVar("MoreSpecific", str, int)

class A(ABC, Generic[Anything]):
    def a_method(self) -> None:
        print("hello")

class B(A[MoreSpecific]):
    pass

class C(B[str]):
    pass

c = C()
c.a_method()  # should NOT emit `no-member` error



""" file suppliermodule.py """
from typing import Optional

class Interface:
    def get_value(self):
        raise NotImplementedError

    def set_value(self, value):
        raise NotImplementedError

class CustomException(Exception): pass

class DoNothing: pass

class DoNothing2: pass

class DoSomething:
    def __init__(
            self,
            a_string: str,
            optional_int: int = None,
            optional_int_2: Optional[int] = None):
        self.my_string = a_string
        self.my_int = optional_int
        self.my_int_2 = optional_int_2

    def do_it(self, new_int: int) -> int:
        return self.my_int + new_int
